# -*- encoding: utf-8 -*-
from abjad.tools.abctools.AbjadObject import AbjadObject


class Dynamic(AbjadObject):
    r'''A dynamic.

    ..  container:: example
    
        **Example 1.** Initializes from dynamic name:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> dynamic = Dynamic('f')
            >>> attach(dynamic, staff[0])

        ..  doctest::

            >>> print format(staff)
            \new Staff {
                c'8 \f
                d'8
                e'8
                f'8
            }

        ::

            >>> show(staff) # doctest: +SKIP

    ..  container:: example

        **Example 2.** Initializes from other dynamic:

        ::

            >>> dynamic_1 = Dynamic('f')
            >>> dynamic_2 = Dynamic(dynamic_1)

        ::

            >>> dynamic_1
            Dynamic('f')

        ::

            >>> dynamic_2
            Dynamic('f')

    '''

    ### CLASS VARIABLES ###

    _default_positional_input_arguments = (
        repr('f'),
        )

    _format_slot = 'right'

    _composite_dynamic_name_to_steady_state_dynamic_name = {
        'fp': 'p', 
        'sf': 'f', 
        'sff': 'ff',
        'sp': 'p', 
        'spp': 'pp',
        'sfz': 'f', 
        'sfp': 'p', 
        'rfz': 'f',
    }

    _dynamic_name_to_dynamic_ordinal = {
        'ppppp': -6, 
        'pppp': -5, 
        'ppp': -4, 
        'pp': -3, 
        'p': -2,
        'mp': -1,
        'mf': 1,
        'f': 2, 
        'ff': 3, 
        'fff': 4, 
        'ffff': 5, 
        'fffff': 6,
        }

    _dynamic_names = (
        'ppppp', 
        'pppp', 
        'ppp', 
        'pp', 
        'p',
        'mp', 
        'mf',
        'f', 
        'ff', 
        'fff', 
        'ffff', 
        'fffff',
        'fp', 
        'sf', 
        'sff', 
        'sp', 
        'spp',
        'sfz', 
        'sfp', 
        'rfz',
        )

    _dynamic_ordinal_to_dynamic_name = {
        -6: 'ppppp', 
        -5: 'pppp', 
        -4: 'ppp', 
        -3: 'pp', 
        -2: 'p',
        -1: 'mp',
        1: 'mf',
        2: 'f', 
        3: 'ff', 
        4: 'fff', 
        5: 'ffff', 
        6: 'fffff',
        }

    ### INITIALIZER ###

    def __init__(self, name):
        from abjad.tools import scoretools
        if isinstance(name, type(self)):
            name = name.name
        self._name = name
        self._default_scope = scoretools.Staff

    ### SPECIAL METHODS ###

    def __copy__(self, *args):
        r'''Copies dynamic.

        Returns new dynamic.
        '''
        return type(self)(self._name)

    def __eq__(self, arg):
        r'''True when `arg` is a dynamic with a name equal to the name of this
        dynamic. Otherwise false.

        Returns boolean.
        '''
        if isinstance(arg, type(self)):
            return self._name == arg._name
        return False

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_repr_string(self):
        return repr(self._name)

    @property
    def _lilypond_format(self):
        return r'\{}'.format(self.name)

    ### PUBLIC PROPERTIES ###


    @property
    def name(self):
        r'''Name of dynamic.

        ::

            >>> dynamic.name
            'f'

        Returns string.
        '''
        return self._name

    ### PUBLIC METHODS ###

    @staticmethod
    def composite_dynamic_name_to_steady_state_dynamic_name(name):
        r'''Changes composite `name` to steady state dynamic name.

        ::

            >>> Dynamic.composite_dynamic_name_to_steady_state_dynamic_name('sfp')
            'p'

        Returns string.
        '''
        return Dynamic._composite_dynamic_name_to_steady_state_dynamic_name[name]

    @staticmethod
    def dynamic_name_to_dynamic_ordinal(name):
        r'''Changes `name` to dynamic ordinal.

        ::

            >>> Dynamic.dynamic_name_to_dynamic_ordinal('fff')
            4

        Returns integer.
        '''
        try:
            return Dynamic._dynamic_name_to_dynamic_ordinal[name]
        except KeyError:
            name = Dynamic.composite_dynamic_name_to_steady_state_dynamic_name(
                name)
            return Dynamic._dynamic_name_to_dynamic_ordinal[name]

    @staticmethod
    def dynamic_ordinal_to_dynamic_name(dynamic_ordinal):
        r'''Changes `dynamic_ordinal` to dynamic name.

        ::

            >>> Dynamic.dynamic_ordinal_to_dynamic_name(-5)
            'pppp'

        Returns string.
        '''
        return Dynamic._dynamic_ordinal_to_dynamic_name[dynamic_ordinal]

    @staticmethod
    def is_dynamic_name(arg):
        r'''True when `arg` is dynamic name. Otherwise false.

        ::

            >>> Dynamic.is_dynamic_name('f')
            True

        Returns boolean.
        '''
        return arg in Dynamic._dynamic_names



# -*- encoding: utf-8 -*-
from abjad import *


def test_pitchtools_OctaveTranspositionMappingInventory_append_01():
    r'''Append named item.
    '''

    inventory = pitchtools.OctaveTranspositionMappingInventory()
    assert repr(inventory) == 'OctaveTranspositionMappingInventory([])'

    mapping = pitchtools.OctaveTranspositionMapping(custom_identifier='foo')
    inventory.append(mapping)
    assert repr(inventory) == \
        "OctaveTranspositionMappingInventory([OctaveTranspositionMapping([], custom_identifier='foo')])"



# -*- encoding: utf-8 -*-
from abjad import *


def test_pitchtools_instantiate_pitch_and_interval_test_collection_01():

    result = pitchtools.instantiate_pitch_and_interval_test_collection()

    assert result == [
        pitchtools.NumberedInversionEquivalentIntervalClass(1),
        pitchtools.NamedInversionEquivalentIntervalClass('M2'),
        pitchtools.NumberedInterval(+1),
        pitchtools.NumberedIntervalClass(+1),
        pitchtools.NamedInterval('+M2'),
        pitchtools.NamedIntervalClass('+M2'),
        pitchtools.NamedPitch('c'),
        pitchtools.NamedPitchClass('c'),
        pitchtools.NumberedPitch(1),
        pitchtools.NumberedPitchClass(1),
        ]



# -*- encoding: utf-8 -*-
from abjad import *


def test_indicatortools_Tempo_list_related_tempos_01():

    tempo = Tempo(Duration(1, 4), 52)
    pairs = tempo.list_related_tempos(
        maximum_numerator=4, 
        maximum_denominator=4,
        )

    assert pairs == [
        (Tempo(Duration(1, 4), 26), mathtools.Ratio(1, 2)), 
        (Tempo(Duration(1, 4), 39), mathtools.Ratio(3, 4)), 
        (Tempo(Duration(1, 4), 52), mathtools.Ratio(1, 1)),
        ]


def test_indicatortools_Tempo_list_related_tempos_02():

    tempo = Tempo(Duration(1, 4), 52)
    pairs = tempo.list_related_tempos(
        maximum_numerator=8, 
        maximum_denominator=8,
        )

    assert pairs == [
        (Tempo(Duration(1, 4), 26), mathtools.Ratio(1, 2)), 
        (Tempo(Duration(1, 4), 39), mathtools.Ratio(3, 4)), 
        (Tempo(Duration(1, 4), 52), mathtools.Ratio(1, 1)), 
        (Tempo(Duration(1, 4), 65), mathtools.Ratio(5, 4)), 
        (Tempo(Duration(1, 4), 78), mathtools.Ratio(3, 2)),
        (Tempo(Duration(1, 4), 91), mathtools.Ratio(7, 4)), 
        (Tempo(Duration(1, 4), 104), mathtools.Ratio(2, 1)),
        ]



# -*- encoding: utf-8 -*-
from abjad import *


def test_pitchtools_sort_named_pitch_carriers_in_expr_01():
    r'''Works on notes.
    '''

    chord = Chord([-12, -10, -2, 4, 8, 11, 17, 19, 27, 30, 33, 37], (1, 4))
    sorted_pitches = pitchtools.sort_named_pitch_carriers_in_expr(
        chord.written_pitches)

    sorted_pitch_numbers = [abs(pitchtools.NumberedPitch(pitch))
        for pitch in sorted_pitches]
    sorted_pcs = [pitchtools.NumberedPitchClass(pitch)
        for pitch in sorted_pitches]

    assert sorted_pitch_numbers == [
        -12, 37, -10, 27, 4, 17, 30, 19, 8, 33, -2, 11]
    assert sorted_pcs == [
        pitchtools.NumberedPitchClass(n) for n in 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]



# -*- encoding: utf-8 -*-
from abjad import *


def test_lilypondfiletools_BookpartBlock_01():

    bookpart_block = lilypondfiletools.BookpartBlock()

    assert systemtools.TestManager.compare(
        bookpart_block,
        r'''
        \bookpart {}
        '''
        )



# -*- encoding: utf-8 -*-
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import detach


# TODO: fix bug in function that causes tied notes to become untied
def replace_contents_of_measures_in_expr(expr, new_contents):
    r'''Replaces contents of measures in `expr` with `new_contents`.

    ..  container:: example

        ::

            >>> staff = Staff(scoretools.make_spacer_skip_measures(
            ...     [(1, 8), (3, 16)]))
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print format(staff)
            \new Staff {
                {
                    \time 1/8
                    s1 * 1/8
                }
                {
                    \time 3/16
                    s1 * 3/16
                }
            }

        ::

            >>> notes = [Note("c'16"), Note("d'16"), Note("e'16"), Note("f'16")]
            >>> scoretools.replace_contents_of_measures_in_expr(staff, notes)
            [Measure((1, 8), "c'16 d'16"), Measure((3, 16), "e'16 f'16 s1 * 1/16")]
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print format(staff)
            \new Staff {
                {
                    \time 1/8
                    c'16
                    d'16
                }
                {
                    \time 3/16
                    e'16
                    f'16
                    s1 * 1/16
                }
            }

    Preserves duration of all measures.

    Skips measures that are too small.

    Pads extra space at end of measures with spacer skip.

    Raises stop iteration if not enough measures.

    Returns measures iterated.
    '''
    from abjad.tools import indicatortools
    from abjad.tools import scoretools

    # init return list
    result = []

    # get first measure and first time signature
    current_measure = scoretools.get_next_measure_from_component(expr)
    result.append(current_measure)
    current_time_signature = current_measure.time_signature
    # to avoide pychecker slice assignment error
    #del(current_measure[:])
    current_measure.__delitem__(slice(0, len(current_measure)))

    # iterate new contents
    while new_contents:

        # find candidate duration of new element plus current measure
        current_element = new_contents[0]
        multiplier = current_time_signature.implied_prolation
        preprolated_duration = current_element._preprolated_duration
        duration = multiplier * preprolated_duration
        candidate_duration = current_measure._get_duration() + duration

        # if new element fits in current measure
        if candidate_duration <= current_time_signature.duration:
            current_element = new_contents.pop(0)
            current_measure._append_without_withdrawing_from_crossing_spanners(
                current_element)

        # otherwise restore current measure and advance to next measure
        else:
            current_time_signature = \
                indicatortools.TimeSignature(current_time_signature)
            detach(indicatortools.TimeSignature, current_measure)
            attach(current_time_signature, current_measure)
            scoretools.append_spacer_skips_to_underfull_measures_in_expr(
                [current_measure])
            current_measure = scoretools.get_next_measure_from_component(
                current_measure)
            if current_measure is None:
                raise StopIteration
            result.append(current_measure)
            current_time_signature = current_measure.time_signature
            # to avoid pychecker slice assignment error
            #del(current_measure[:])
            current_measure.__delitem__(slice(0, len(current_measure)))

    # restore last iterated measure
    current_time_signature = \
        indicatortools.TimeSignature(current_time_signature)
    detach(indicatortools.TimeSignature, current_measure)
    attach(current_time_signature, current_measure)
    scoretools.append_spacer_skips_to_underfull_measures_in_expr(
        current_measure)

    # return iterated measures
    return result



# -*- encoding: utf-8 -*-
from abjad.tools import sequencetools


def list_numbered_inversion_equivalent_interval_classes_pairwise(pitch_carriers, wrap=False):
    r'''List numbered inversion-equivalent interval-classes pairwise between
    `pitch_carriers`:

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            c'8
            d'8
            e'8
            f'8
            g'8
            a'8
            b'8
            c''8
        }

    ::

        >>> result = pitchtools.list_numbered_inversion_equivalent_interval_classes_pairwise(
        ... staff, wrap=False)

    ::

        >>> for x in result: x
        ...
        NumberedInversionEquivalentIntervalClass(2)
        NumberedInversionEquivalentIntervalClass(2)
        NumberedInversionEquivalentIntervalClass(1)
        NumberedInversionEquivalentIntervalClass(2)
        NumberedInversionEquivalentIntervalClass(2)
        NumberedInversionEquivalentIntervalClass(2)
        NumberedInversionEquivalentIntervalClass(1)

    ::

        >>> result = pitchtools.list_numbered_inversion_equivalent_interval_classes_pairwise(
        ... staff, wrap=True)

    ::

        >>> for x in result: x
        NumberedInversionEquivalentIntervalClass(2)
        NumberedInversionEquivalentIntervalClass(2)
        NumberedInversionEquivalentIntervalClass(1)
        NumberedInversionEquivalentIntervalClass(2)
        NumberedInversionEquivalentIntervalClass(2)
        NumberedInversionEquivalentIntervalClass(2)
        NumberedInversionEquivalentIntervalClass(1)
        NumberedInversionEquivalentIntervalClass(0)

    ::

        >>> notes = staff.select_leaves()
        >>> notes = list(reversed(notes))

    ::

        >>> result = pitchtools.list_numbered_inversion_equivalent_interval_classes_pairwise(
        ... notes, wrap=False)

    ::

        >>> for x in result: x
        ...
        NumberedInversionEquivalentIntervalClass(1)
        NumberedInversionEquivalentIntervalClass(2)
        NumberedInversionEquivalentIntervalClass(2)
        NumberedInversionEquivalentIntervalClass(2)
        NumberedInversionEquivalentIntervalClass(1)
        NumberedInversionEquivalentIntervalClass(2)
        NumberedInversionEquivalentIntervalClass(2)

    ::

        >>> result = pitchtools.list_numbered_inversion_equivalent_interval_classes_pairwise(
        ... notes, wrap=True)

    ::

        >>> for x in result: x
        ...
        NumberedInversionEquivalentIntervalClass(1)
        NumberedInversionEquivalentIntervalClass(2)
        NumberedInversionEquivalentIntervalClass(2)
        NumberedInversionEquivalentIntervalClass(2)
        NumberedInversionEquivalentIntervalClass(1)
        NumberedInversionEquivalentIntervalClass(2)
        NumberedInversionEquivalentIntervalClass(2)
        NumberedInversionEquivalentIntervalClass(0)

    When ``wrap=False`` do not return ``pitch_carriers[-1] - pitch_carriers[0]`` as last in series.

    When ``wrap=True`` do return ``pitch_carriers[-1] - pitch_carriers[0]`` as last in series.

    Returns list.
    '''
    from abjad.tools import pitchtools

    result = []

    if len(pitch_carriers) == 0:
        return result
    elif len(pitch_carriers) == 1:
        if pitchtools.Pitch.is_pitch_carrier(pitch_carriers[0]):
            return result
        else:
            message = 'must be pitch, note, note-head or chord.'
            raise TypeError(message)

    if wrap:
        pairs = sequencetools.iterate_sequence_pairwise_wrapped(pitch_carriers)
    else:
        pairs = sequencetools.iterate_sequence_pairwise_strict(pitch_carriers)

    for first_carrier, second_carrier in pairs:
        first_pitch = pitchtools.get_named_pitch_from_pitch_carrier(first_carrier)
        second_pitch = pitchtools.get_named_pitch_from_pitch_carrier(second_carrier)
        mdi = second_pitch - first_pitch
        iecic = pitchtools.NumberedInversionEquivalentIntervalClass(mdi)
        result.append(iecic)

    return result



# -*- encoding: utf-8 -*-
from abjad import *


def test_systemtools_AbjadConfiguration_list_abjad_environment_variables_01():

    environment_variables = \
        systemtools.AbjadConfiguration.list_abjad_environment_variables()

    assert all(isinstance(x, tuple) for x in environment_variables)



# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools import sequencetools


def test_sequencetools_all_are_unequal_01():
    r'''True when the elements in input iterable are unique.
    '''

    assert sequencetools.all_are_unequal([1, 2, 3])


def test_sequencetools_all_are_unequal_02():
    r'''False when the elements in input iterable are not unique.
    '''

    assert not sequencetools.all_are_unequal([1, 1, 1, 2, 3])


def test_sequencetools_all_are_unequal_03():
    r'''False when expr is not a sequence.
    '''

    assert not sequencetools.all_are_unequal(17)



def test_ly_environment_01():

    from abjad import ly

    assert ly.context_properties is not None
    print ly.context_properties

    assert ly.contexts is not None
    print ly.contexts

    assert ly.current_module is not None
    print ly.current_module

    assert ly.engravers is not None
    print ly.engravers

    assert ly.grob_interfaces is not None
    print ly.grob_interfaces

    assert ly.interface_properties is not None
    print ly.interface_properties

    assert ly.language_pitch_names is not None
    print ly.language_pitch_names

    assert ly.markup_functions is not None
    print ly.markup_functions

    assert ly.markup_list_functions is not None
    print ly.markup_list_functions

    assert ly.music_glyphs is not None
    print ly.music_glyphs



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools


def join_subsequences(sequence):
    '''Join subsequences in `sequence`:

    ::

        >>> sequencetools.join_subsequences([(1, 2, 3), (), (4, 5), (), (6,)])
        (1, 2, 3, 4, 5, 6)

    Returns newly constructed object of subsequence type.
    '''

    return mathtools.cumulative_sums(sequence, start=None)[-1]



# -*- encoding: utf-8 -*-
'''Time signature manipulation tools.

   Modules in this package depend on, may freely import from, the following:

      from abjad.tools import durationtools
      from abjad.tools import mathtools
'''

from abjad.tools import systemtools

systemtools.ImportManager.import_structured_package(
	__path__[0],
	globals(),
	)

_documentation_section = 'core'



# -*- encoding: utf-8 -*-
from abjad.tools.datastructuretools.TypedList import TypedList
from abjad.tools.pitchtools.OctaveTranspositionMapping \
    import OctaveTranspositionMapping


class OctaveTranspositionMappingInventory(TypedList):
    '''An ordered list of octave transposition mappings.

    ::

        >>> mapping_1 = pitchtools.OctaveTranspositionMapping(
        ...     [('[A0, C4)', 15), ('[C4, C8)', 27)])
        >>> mapping_2 = pitchtools.OctaveTranspositionMapping(
        ...     [('[A0, C8]', -18)])
        >>> inventory = pitchtools.OctaveTranspositionMappingInventory(
        ...     [mapping_1, mapping_2])

    ::

        >>> print format(inventory)
        pitchtools.OctaveTranspositionMappingInventory(
            [
                pitchtools.OctaveTranspositionMapping(
                    [
                        pitchtools.OctaveTranspositionMappingComponent(
                            pitchtools.PitchRange(
                                '[A0, C4)'
                                ),
                            pitchtools.NumberedPitch(15)
                            ),
                        pitchtools.OctaveTranspositionMappingComponent(
                            pitchtools.PitchRange(
                                '[C4, C8)'
                                ),
                            pitchtools.NumberedPitch(27)
                            ),
                        ]
                    ),
                pitchtools.OctaveTranspositionMapping(
                    [
                        pitchtools.OctaveTranspositionMappingComponent(
                            pitchtools.PitchRange(
                                '[A0, C8]'
                                ),
                            pitchtools.NumberedPitch(-18)
                            ),
                        ]
                    ),
                ]
            )

    Octave transposition mapping inventories implement list interface
    and are mutable.
    '''

    ### PRIVATE PROPERTIES ###

    @property
    def _item_callable(self):
        return OctaveTranspositionMapping



# -*- encoding: utf-8 -*-


def iterate_sequence_pairwise_wrapped(sequence):
    '''Iterate `sequence` pairwise wrapped:

    ::

        >>> list(sequencetools.iterate_sequence_pairwise_wrapped(range(6)))
        [(0, 1), (1, 2), (2, 3), (3, 4), (4, 5), (5, 0)]

    Returns pair generator.
    '''

    for i in range(len(sequence)):
        yield (sequence[i], sequence[(i + 1) % len(sequence)])



# -*- encoding: utf-8 -*-
import copy
from abjad import *


def test_quantizationtools_QGridLeaf___copy___01():

    leaf = quantizationtools.QGridLeaf(1)

    copied = copy.copy(leaf)

    assert leaf == copied
    assert leaf is not copied


def test_quantizationtools_QGridLeaf___copy___02():

    leaf = quantizationtools.QGridLeaf(2, [
        quantizationtools.QEventProxy(quantizationtools.SilentQEvent(1000), 0.5)
        ])

    copied = copy.copy(leaf)

    assert leaf == copied
    assert leaf is not copied



# -*- encoding: utf-8 -*-
import os
from abjad.tools.abctools import AbjadObject


class ModuleCrawler(AbjadObject):
    r'''Crawls `code_root`, yielding all module objects whose name begins with
    `root_package_name`.

    Return `ModuleCrawler` instance.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_code_root',
        '_ignored_directory_names',
        '_root_package_name',
        '_visit_private_modules',
        )

    ### INITIALIZER ###

    def __init__(self,
        code_root=None,
        ignored_directory_names=(
            '__pycache__',
            '.git',
            '.svn',
            'test',
            ),
        root_package_name=None,
        visit_private_modules=False,
        ):
        from abjad import abjad_configuration
        if code_root is None:
            code_root = abjad_configuration.abjad_directory_path
        assert os.path.exists(code_root)
        if not os.path.exists(os.path.join(code_root, '__init__.py')):
            message = '{} is not a Python package directory.'
            message = message.format(code_root)
            raise ValueError(message)
        code_root = os.path.abspath(code_root)

        if root_package_name is None:
            parts = code_root.split(os.path.sep)
            root_package_name = parts[-1]
            while os.path.exists(os.path.join(
                os.path.sep.join(parts), '__init__.py')):
                root_package_name = parts.pop()

        self._code_root = code_root
        self._ignored_directory_names = ignored_directory_names
        self._root_package_name = root_package_name
        self._visit_private_modules = bool(visit_private_modules)

    ### SPECIAL METHODS ###

    def __iter__(self):
        r'''Iterates module crawler.

        Returns generator.
        '''
        assert os.path.exists(self.code_root)

        if not os.path.exists(os.path.join(self.code_root, '__init__.py')):
            return

        for current_root, directories, files in os.walk(self.code_root):

            # filter directories
            for directory in directories[:]:
                if directory in self.ignored_directory_names:
                    directories.remove(directory)
                elif directory.startswith('_') and \
                    not self.visit_private_modules:
                    directories.remove(directory)
                elif not os.path.exists(os.path.join(
                    current_root, directory, '__init__.py')):
                    directories.remove(directory)
            directories.sort()

            # filter files
            for file in files[:]:
                if file.startswith('__'):
                    files.remove(file)
                elif not file.endswith('.py'):
                    files.remove(file)
            files.sort()

            # process files
            for file in files:
                path = os.path.join(current_root, file).replace('.py', '')
                parts = path.split(os.path.sep)
                object_name = parts[-1]
                module_name = []
                for part in reversed(parts):
                    module_name.append(part)
                    if part == self.root_package_name:
                        break
                module_name = '.'.join(reversed(module_name))
                module = __import__(module_name, fromlist=['*'])
                yield module

    ### PUBLIC PROPERTIES ###

    @property
    def code_root(self):
        r'''Code root of module crawler.

        Returns string.
        '''
        return self._code_root

    @property
    def ignored_directory_names(self):
        r'''Ignored directory names of module crawler.

        Returns tuple.
        '''
        return self._ignored_directory_names

    @property
    def root_package_name(self):
        r'''Root package name of module crawler.

        Returns string.
        '''
        return self._root_package_name

    @property
    def visit_private_modules(self):
        r'''Visit private modules.

        Returns boolean.
        '''
        return self._visit_private_modules



# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Container___setitem___integer_01():
    r'''Spanned leaves exchange correctly.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, voice[:2])
    glissando = spannertools.Glissando()
    attach(glissando, voice.select_leaves())

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'8 [ \glissando
            d'8 ] \glissando
            e'8 \glissando
            f'8
        }
        '''
        )

    voice[1] = Note(12, (1, 8))

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'8 [ \glissando
            c''8 ] \glissando
            e'8 \glissando
            f'8
        }
        '''
        )

    assert inspect(voice).is_well_formed()


def test_scoretools_Container___setitem___integer_02():
    r'''Spanned leaf hands position over to container correctly.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, voice[:2])
    glissando = spannertools.Glissando()
    attach(glissando, voice.select_leaves())

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'8 [ \glissando
            d'8 ] \glissando
            e'8 \glissando
            f'8
        }
        '''
        )

    voice[1] = Container(scoretools.make_repeated_notes(3, Duration(1, 16)))

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'8 [ \glissando
            {
                c'16 \glissando
                c'16 \glissando
                c'16 ] \glissando
            }
            e'8 \glissando
            f'8
        }
        '''
        )

    assert inspect(voice).is_well_formed()


def test_scoretools_Container___setitem___integer_03():
    r'''Directly spanned contains hand over correctly to a single leaf.
    Note here that only the sequentials are initially spanned.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 }")
    beam = Beam()
    attach(beam, voice[:])
    glissando = spannertools.Glissando()
    attach(glissando, voice[:])

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            {
                c'8 [ \glissando
                d'8 \glissando
            }
            {
                e'8 \glissando
                f'8 ]
            }
        }
        '''
        )

    voice[1] = Note(12, (1, 8))

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            {
                c'8 [ \glissando
                d'8 \glissando
            }
            c''8 ]
        }
        '''
        )

    assert inspect(voice).is_well_formed()


def test_scoretools_Container___setitem___integer_04():
    r'''Indirectly spanned containers hand over correctly to a single leaf.
    Notice here that only LEAVES are initially spanned.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 }")
    beam = Beam()
    attach(beam, voice.select_leaves())
    glissando = spannertools.Glissando()
    attach(glissando, voice.select_leaves())

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            {
                c'8 [ \glissando
                d'8 \glissando
            }
            {
                e'8 \glissando
                f'8 ]
            }
        }
        '''
        )

    voice[1] = Note(12, (1, 8))

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            {
                c'8 [ \glissando
                d'8 \glissando
            }
            c''8 ]
        }
        '''
        )

    assert inspect(voice).is_well_formed()


def test_scoretools_Container___setitem___integer_05():
    r'''Directly spanned containers hand over to other containers correctly.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 }")
    beam = Beam()
    attach(beam, voice[:])
    glissando = spannertools.Glissando()
    attach(glissando, voice[:])

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            {
                c'8 [ \glissando
                d'8 \glissando
            }
            {
                e'8 \glissando
                f'8 ]
            }
        }
        '''
        )

    voice[1] = scoretools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            {
                c'8 [ \glissando
                d'8 \glissando
            }
            \times 2/3 {
                c'8 \glissando
                d'8 \glissando
                e'8 ]
            }
        }
        '''
        )

    assert inspect(voice).is_well_formed()


def test_scoretools_Container___setitem___integer_06():
    r'''Indirectly spanned containers hand over correctly to a single leaf.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 }")
    beam = Beam()
    attach(beam, voice.select_leaves())
    glissando = spannertools.Glissando()
    attach(glissando, voice.select_leaves())

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            {
                c'8 [ \glissando
                d'8 \glissando
            }
            {
                e'8 \glissando
                f'8 ]
            }
        }
        '''
        )

    voice[1] = Note(12, (1, 8))

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            {
                c'8 [ \glissando
                d'8 \glissando
            }
            c''8 ]
        }
        '''
        )

    assert inspect(voice).is_well_formed()


def test_scoretools_Container___setitem___integer_07():
    r'''Indirectly HALF-spanned containers hand over correctly to a
    single leaf. WOW!'''

    voice = Voice(Container(scoretools.make_repeated_notes(4)) * 2)
    voice = Voice("{ c'8 d'8 e'8 f'8 } { g'8 a'8 b'8 c''8 }")
    beam = Beam()
    attach(beam, voice.select_leaves()[0:6])

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            {
                c'8 [
                d'8
                e'8
                f'8
            }
            {
                g'8
                a'8 ]
                b'8
                c''8
            }
        }
        '''
        )

    voice[1] = Rest((1, 2))

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            {
                c'8 [
                d'8
                e'8
                f'8 ]
            }
            r2
        }
        '''
        )

    assert inspect(voice).is_well_formed()


def test_scoretools_Container___setitem___integer_08():
    r'''Take spanned leaf from donor container
    and insert into recipient container.
    Both donor and recipient check after set item.
    '''

    notes = [Note("c'8"), Note("d'8"), Note("e'8"), Note("f'8"), Note("g'8"), Note("a'8")]

    voice = Voice(notes[:3])
    beam = Beam()
    attach(beam, voice[:])

    r'''
    \new Voice {
        c'8 [
        d'8
        e'8 ]
    }
    '''

    u = Voice(notes[3:])
    beam = Beam()
    attach(beam, u[:])

    r'''
    \new Voice {
        f'8 [
        g'8
        a'8 ]
    }
    '''

    voice[1] = u[1]

    "Modified voice:"

    r'''
    \new Voice {
        c'8 [
        g'8
        e'8 ]
    }
    '''

    assert inspect(voice).is_well_formed()

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'8 [
            g'8
            e'8 ]
        }
        '''
        )

    "Modified u:"

    assert systemtools.TestManager.compare(
        u,
        r'''
        \new Voice {
            f'8 [
            a'8 ]
        }
        '''
        )

    assert inspect(u).is_well_formed()


def test_scoretools_Container___setitem___integer_09():
    r'''Take down-spanned container with completely covered spanner
    from donor container and insert into recipient container.
    Both donor and recipient check after set item.
    '''

    notes = [
        Note("c'8"), Note("d'8"), Note("e'8"), 
        Note("f'8"), Note("g'8"), Note("a'8"), Note("b'8"),
        ]
    voice = Voice(notes[:3])
    beam = Beam()
    attach(beam, voice[:])

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'8 [
            d'8
            e'8 ]
        }
        '''
        )

    u = Voice(notes[3:])
    Container(u[1:3])
    glissando = spannertools.Glissando()
    attach(glissando, u.select_leaves())
    slur = Slur()
    attach(slur, u[1].select_leaves())

    assert systemtools.TestManager.compare(
        u,
        r'''
        \new Voice {
            f'8 \glissando
            {
                g'8 \glissando (
                a'8 \glissando )
            }
            b'8
        }
        '''
        )

    voice[1] = u[1]

    "Voice voice is now ..."

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'8 [
            {
                g'8 (
                a'8 )
            }
            e'8 ]
        }
        '''
        )

    assert inspect(voice).is_well_formed()

    "Voice u is now ..."

    assert inspect(u).is_well_formed()
    assert systemtools.TestManager.compare(
        u,
        r'''
        \new Voice {
            f'8 \glissando
            b'8
        }
        '''
        )



# -*- encoding: utf-8 -*-
import abc
import types


class StorageFormatManager(object):
    r'''Manages Abjad object storage formats.
    '''

    ### PUBLIC METHODS ###

    @staticmethod
    def compare(object_one, object_two):
        r'''Compares `object_one` to `object_two`.

        Returns boolean.
        '''
        if type(object_one) is not type(object_two):
            return False
        if StorageFormatManager.get_positional_argument_values(object_one) != \
            StorageFormatManager.get_positional_argument_values(object_two):
            return False
        if StorageFormatManager.get_keyword_argument_values(object_one) != \
            StorageFormatManager.get_keyword_argument_values(object_two):
            return False
        return True

    @staticmethod
    def format_one_value(
        value,
        as_storage_format=True,
        is_indented=True,
        ):
        r'''Formats one value.

        Returns list.
        '''
        result = []
        prefix, infix, suffix = StorageFormatManager.get_indentation_strings(
            is_indented)
        if isinstance(value, types.MethodType):
            return result
        if type(value) is abc.ABCMeta:
            if as_storage_format:
                value = '{}.{}'.format(
                    StorageFormatManager.get_tools_package_name(value),
                    value.__name__,
                    )
            else:
                value = value.__name__
            result.append(value)
        elif as_storage_format and hasattr(
            value, '_storage_format_specification'):
            specification = value._storage_format_specification
            pieces = StorageFormatManager.get_format_pieces(
                specification,
                as_storage_format=True,
                )
            result.extend(pieces)
        elif not as_storage_format and hasattr(
            value, '_repr_specification'):
            specification = value._repr_specification
            pieces = StorageFormatManager.get_format_pieces(
                specification,
                as_storage_format=False,
                )
            result.extend(pieces)
        elif isinstance(value, (list, tuple)):
            # just return the repr, if all contents are builtin types
            if all(isinstance(x, (bool, int, float, str, type(None)))
                for x in value):
                piece = repr(value)
                if len(piece) < 50:
                    return [repr(value)]
            if isinstance(value, list):
                braces = '[', ']'
            else:
                braces = '(', ')'
            result.append('{}{}'.format(braces[0], infix))
            for x in value:
                pieces = StorageFormatManager.format_one_value(
                    x,
                    as_storage_format=as_storage_format,
                    is_indented=is_indented,
                    )
                for piece in pieces[:-1]:
                    result.append('{}{}'.format(prefix, piece))
                result.append('{}{}{}'.format(prefix, pieces[-1], suffix))
            if not is_indented:
                if isinstance(value, list) or 1 < len(value):
                    result[-1] = result[-1].rstrip(suffix)
                else:
                    result[-1] = result[-1].rstrip()
            result.append('{}{}'.format(prefix, braces[1]))
        elif isinstance(value, dict):
            result.append('{{{}'.format(infix))
            for key, value in sorted(value.items()):
                key_pieces = StorageFormatManager.format_one_value(
                    key,
                    as_storage_format=as_storage_format,
                    is_indented=is_indented,
                    )
                value_pieces = StorageFormatManager.format_one_value(
                    value,
                    as_storage_format=as_storage_format,
                    is_indented=is_indented,
                    )
                for x in key_pieces[:-1]:
                    result.append('{}{}'.format(prefix, x))
                result.append('{}{}: {}'.format(
                    prefix, key_pieces[-1], value_pieces[0]))
                for x in value_pieces[1:]:
                    result.append('{}{}'.format(prefix, x))
                result[-1] = '{}{}'.format(result[-1], suffix)
            if not is_indented:
                result[-1] = result[-1].rstrip(suffix) + infix
            result.append('{}}}'.format(prefix))
        else:
            result.append(repr(value))
        return result

    @staticmethod
    def get_format_pieces(
        specification,
        as_storage_format=True,
        ):
        r'''Gets format pieces.
        '''
        if specification.storage_format_pieces is not None:
            return specification.storage_format_pieces

        result = []
        prefix, infix, suffix = StorageFormatManager.get_indentation_strings(
            specification.is_indented)

        class_name = type(specification.instance).__name__
        if as_storage_format:
            tools_package_name = specification.tools_package_name
            class_name_prefix = '{}.{}'.format(
                tools_package_name, class_name)
        else:
            class_name_prefix = class_name

        positional_argument_pieces = []
        for value in specification.positional_argument_values:
            pieces = StorageFormatManager.format_one_value(
                value,
                as_storage_format=as_storage_format,
                is_indented=specification.is_indented,
                )
            for piece in pieces[:-1]:
                positional_argument_pieces.append(prefix + piece)
            positional_argument_pieces.append(prefix + pieces[-1] + suffix)

        keyword_argument_pieces = []
        for name in specification.keyword_argument_names:
            value = getattr(specification.instance, name)
            if value is None or isinstance(value, types.MethodType):
                continue
            pieces = StorageFormatManager.format_one_value(
                value,
                as_storage_format=as_storage_format,
                is_indented=specification.is_indented,
                )
            pieces[0] = '{}={}'.format(name, pieces[0])
            for piece in pieces[:-1]:
                keyword_argument_pieces.append(prefix + piece)
            keyword_argument_pieces.append(prefix + pieces[-1] + suffix)

        if not as_storage_format and specification.is_bracketted:
            result.append('<')

        if not as_storage_format and specification.body_text:
            result.append('{}({})'.format(
                class_name_prefix,
                specification.body_text,
                ))
        else:
            if not positional_argument_pieces and not keyword_argument_pieces:
                result.append('{}()'.format(class_name_prefix))
            else:
                result.append('{}({}'.format(
                    class_name_prefix,
                    infix,
                    ))
                result.extend(positional_argument_pieces)
                if positional_argument_pieces and not keyword_argument_pieces:
                    result[-1] = result[-1].rstrip(suffix) + infix
                else:
                    result.extend(keyword_argument_pieces)
                if not as_storage_format:
                    result[-1] = result[-1].rstrip(suffix) + infix
                if specification.is_indented:
                    result.append('{})'.format(prefix))
                else:
                    result.append(')')

        if not as_storage_format and specification.is_bracketted:
            result.append('>')

        if not specification.is_indented:
            return (''.join(result),)
        return tuple(result)

    @staticmethod
    def get_indentation_strings(is_indented):
        r'''Gets indentation strings.
        '''
        prefix, infix, suffix = '', '', ', '
        if is_indented:
            prefix, infix, suffix = '    ', '\n', ',\n'
        return prefix, infix, suffix

    @staticmethod
    def get_input_argument_values(object_):
        r'''Gets input argument values.
        '''
        return StorageFormatManager.get_positional_argument_values(object_) + \
            StorageFormatManager.get_keyword_argument_values(object_)

    @staticmethod
    def get_keyword_argument_dictionary(object_):
        r'''Gets keyword argument dictionary.
        '''
        names = StorageFormatManager.get_keyword_argument_names(object_)
        values = StorageFormatManager.get_keyword_argument_values(object_)
        assert len(names) == len(values)
        result = dict(zip(names, values))
        return result

    @staticmethod
    def get_keyword_argument_names(object_):
        r'''Gets keyword argument names.
        '''
        return StorageFormatManager.get_signature_keyword_argument_names(
            object_)

    @staticmethod
    def get_keyword_argument_values(object_):
        r'''Gets keyword argument values.
        '''
        result = []
        for name in StorageFormatManager.get_keyword_argument_names(object_):
            result.append(getattr(object_, name))
        return tuple(result)

    @staticmethod
    def get_positional_argument_dictionary(object_):
        r'''Gets positional argument dictionary.
        '''
        names = StorageFormatManager.get_positional_argument_names(object_)
        values = StorageFormatManager.get_positional_argument_values(object_)
        assert len(names) == len(values)
        result = dict(zip(names, values))
        return result

    @staticmethod
    def get_positional_argument_names(object_):
        r'''Gets positional argument names.
        '''
        return StorageFormatManager.get_signature_positional_argument_names(
            object_)

    @staticmethod
    def get_positional_argument_values(object_):
        r'''Gets positional argument values.
        '''
        names = StorageFormatManager.get_positional_argument_names(object_)
        result = []
        for name in names:
            result.append(getattr(object_, name))
        return tuple(result)

    @staticmethod
    def get_repr_format(
        object_,
        ):
        r'''Gets interpreter representation format.
        '''
        assert '_repr_specification' in dir(object_)
        specification = object_._repr_specification
        pieces = StorageFormatManager.get_format_pieces(
            specification,
            as_storage_format=False,
            )
        return ''.join(pieces)

    @staticmethod
    def get_signature_keyword_argument_names(object_):
        r'''Gets signature keyword argument names.
        '''
        if hasattr(object_.__init__, '__func__'):
            initializer = object_.__init__.__func__
            if initializer.func_defaults:
                keyword_argument_count = len(initializer.func_defaults)
                initializer_code = initializer.func_code
                positional_argument_count = (
                    initializer_code.co_argcount - keyword_argument_count - 1)
                start_index = 1 + positional_argument_count
                stop_index = start_index + keyword_argument_count
                return initializer_code.co_varnames[start_index:stop_index]
            else:
                return ()
        return ()

    @staticmethod
    def get_signature_positional_argument_names(object_):
        r'''Gets signature positional argument names.
        '''
        if hasattr(object_.__init__, '__func__'):
            initializer = object_.__init__.__func__
            if initializer.func_defaults:
                keyword_argument_count = len(initializer.func_defaults)
            else:
                keyword_argument_count = 0
            initializer_code = initializer.func_code
            positional_argument_count = (
                initializer_code.co_argcount - keyword_argument_count - 1)
            start_index, stop_index = 1, 1 + positional_argument_count
            return initializer_code.co_varnames[start_index:stop_index]
        return ()

    @staticmethod
    def get_storage_format(
        object_,
        ):
        r'''Gets storage format.
        '''
        assert '_storage_format_specification' in dir(object_)
        specification = object_._storage_format_specification
        pieces = StorageFormatManager.get_format_pieces(
            specification,
            as_storage_format=True,
            )
        return ''.join(pieces)

    @staticmethod
    def get_tools_package_name(object_):
        r'''Gets tools-package name of `object_`.

        ::

            >>> manager = systemtools.StorageFormatManager
            >>> manager.get_tools_package_name(Note)
            'scoretools'

        '''
        if StorageFormatManager.is_instance(object_):
            class_name = type(object_).__name__
        else:
            class_name = object_.__name__
        for part in reversed(object_.__module__.split('.')):
            if not part == class_name:
                return part

    @staticmethod
    def get_tools_package_qualified_class_name(object_):
        r'''Gets tools-package qualified class name of `object_`.

        ::

            >>> manager = systemtools.StorageFormatManager
            >>> manager.get_tools_package_qualified_class_name(Note)
            'scoretools.Note'

        Returns string.
        '''
        tools_package_name = None
        if StorageFormatManager.is_instance(object_):
            class_name = type(object_).__name__
        else:
            class_name = object_.__name__
        if not tools_package_name:
            for part in reversed(object_.__module__.split('.')):
                if not part == class_name:
                    tools_package_name = part
                    break
        return '{}.{}'.format(tools_package_name, class_name)

    @staticmethod
    def is_instance(object_):
        r'''True when `object_` is instance. Otherwise false.

        Returns boolean.
        '''
        if isinstance(object_, types.TypeType):
            return False
        elif type(object_) is object_.__class__:
            return True
        return False



# -*- encoding: utf-8 -*-
from abjad import *


def test_scoretools_NoteHeadInventory___setitem___01():
    r'''Set note head with pitch number.
    '''

    chord = Chord("<c' d'>4")
    chord.note_heads[1] = 4

    assert format(chord) == "<c' e'>4"


def test_scoretools_NoteHeadInventory___setitem___02():
    '''Set note head with pitch.
    '''

    chord = Chord("<c' d'>4")
    chord.note_heads[1] = NamedPitch("e'")

    assert format(chord) == "<c' e'>4"


def test_scoretools_NoteHeadInventory___setitem___03():
    r'''Set note head with tweaked note head.
    '''

    chord = Chord("<c' cs'' f''>4")
    note_head = scoretools.NoteHead(3)
    note_head.tweak.color = 'red'
    chord.note_heads[0] = note_head

    assert systemtools.TestManager.compare(
        chord,
        r'''
        <
            \tweak #'color #red
            ef'
            cs''
            f''
        >4
        '''
        )

    assert inspect(chord).is_well_formed()



# -*- encoding: utf-8 -*-


def string_to_space_delimited_lowercase(string):
    r'''Change uppercamelcase `string` to space-delimited lowercase:

    ::

        >>> stringtools.string_to_space_delimited_lowercase('LogicalTie')
        'logical tie'

    Change underscore-delimited `string` to space-delimited lowercase:

    ::

        >>> stringtools.string_to_space_delimited_lowercase('logical_tie')
        'logical tie'

    Returns space-delimited string unchanged:

    ::

        >>> stringtools.string_to_space_delimited_lowercase('logical tie')
        'logical tie'

    Returns empty `string` unchanged:

    ::

        >>> stringtools.string_to_space_delimited_lowercase('')
        ''

    Returns string.
    '''
    from abjad.tools import stringtools

    if not string:
        return string
    elif string[0].isupper():
        return stringtools.upper_camel_case_to_space_delimited_lowercase(string)
    else:
        return string.replace('_', ' ')



# -*- encoding: utf-8 -*-


def arg_to_tridirectional_direction_string(arg):
    r'''Convert `arg` to tridirectional direction string:

        >>> stringtools.arg_to_tridirectional_direction_string('^')
        'up'

    ::

        >>> stringtools.arg_to_tridirectional_direction_string('-')
        'center'

    ::

        >>> stringtools.arg_to_tridirectional_direction_string('_')
        'down'

    ::

        >>> stringtools.arg_to_tridirectional_direction_string(1)
        'up'

    ::

        >>> stringtools.arg_to_tridirectional_direction_string(0)
        'center'

    ::

        >>> stringtools.arg_to_tridirectional_direction_string(-1)
        'down'

    ::

        >>> stringtools.arg_to_tridirectional_direction_string('default')
        'center'

    If `arg` is None, None will be returned.

    Returns str or None.
    '''

    lookup = {
        Up: 'up',
        '^': 'up',
        'up': 'up',
        1: 'up',
        Down: 'down',
        '_': 'down',
        'down': 'down',
        -1: 'down',
        Center: 'center',
        '-': 'center',
        0: 'center',
        'center': 'center',
        'default': 'center',
        'neutral': 'center',
    }

    if arg is None:
        return None
    elif arg in lookup:
        return lookup[arg]
    raise ValueError



# -*- encoding: utf-8 -*-


def difference_series(sequence):
    r'''Difference series of `sequence`.

    ::

        >>> mathtools.difference_series([1, 1, 2, 3, 5, 5, 6])
        [0, 1, 1, 2, 0, 1]

    Returns list.
    '''

    result = []

    for i, n in enumerate(sequence[1:]):
        result.append(n - sequence[i])

    return result



# -*- encoding: utf-8 -*-
import inspect
import pickle
import pytest
import abjad
from abjad.tools import documentationtools
pytest.skip()


classes = documentationtools.list_all_abjad_classes()
@pytest.mark.parametrize('class_', classes)
def test_pickle_01(class_):
    r'''All storage-formattable classes are pickable.
    '''

    if '_storage_format_specification' in dir(class_) and \
        not inspect.isabstract(class_):
        if hasattr(class_, '_default_positional_input_arguments'):
            args = class_._default_positional_input_arguments
            instance_one = class_(*args)
        else:
            instance_one = class_()
        pickle_string = pickle.dumps(instance_one)
        instance_two = pickle.loads(pickle_string)
        instance_one_format = format(instance_one, 'storage')
        instance_two_format = format(instance_two, 'storage')
        assert instance_one_format == instance_two_format



# -*- encoding: utf-8 -*-
import time
from abjad.tools.abctools import AbjadObject


class DateTimeToken(AbjadObject):
    '''Date / time token.

    ::

        >>> lilypondfiletools.DateTimeToken()
        DateTimeToken(...)

    Returns date / time token.
    '''

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Gets format.

        Returns string.
        '''
        if format_specification in ('', 'lilypond'):
            return self._lilypond_format
        return str(self)

    def __repr__(self):
        r'''Gets interpreter representation of date / time token.

        Returns string.
        '''
        return '{}({})'.format(type(self).__name__, self._lilypond_format)

    ### PRIVATE PROPERTIES ###

    @property
    def _lilypond_format(self):
        current_time_string = time.strftime('%Y-%m-%d %H:%M')
        return current_time_string



# -*- encoding: utf-8 -*-
from abjad.tools.schemetools.Scheme import Scheme


class SchemeVector(Scheme):
    '''Abjad model of Scheme vector:

    ::

        >>> schemetools.SchemeVector(True, True, False)
        SchemeVector(True, True, False)

    Scheme vectors and Scheme vector constants differ in only 
    their LilyPond input format.

    Scheme vectors are immutable.
    '''

    ### CLASS VARIABLES ##

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(self, *args):
        Scheme.__init__(self, *args, **{'quoting': "'"})



# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Container_extend_01():
    r'''Extend container with list of leaves.
    '''

    voice = Voice("c'8 d'8")
    beam = Beam()
    attach(beam, voice[:])

    r'''
    \new Voice {
        c'8 [
        d'8 ]
    }
    '''

    voice.extend([Note("c'8"), Note("d'8")])

    r'''
    \new Voice {
        c'8 [
        d'8 ]
        c'8
        d'8
    }
    '''

    assert inspect(voice).is_well_formed()
    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'8 [
            d'8 ]
            c'8
            d'8
        }
        '''
        )


def test_scoretools_Container_extend_02():
    r'''Extend container with contents of other container.
    '''

    voice = Voice("c'8 d'8")
    beam = Beam()
    attach(beam, voice[:])

    r'''
    \new Voice {
        c'8 [
        d'8 ]
    }
    '''

    u = Voice([Note(4, (1, 8)), Note(5, (1, 8))])
    beam = Beam()
    attach(beam, u[:])
    voice.extend(u)

    r'''
    \new Voice {
        c'8 [
        d'8 ]
        e'8 [
        f'8 ]
    }
    '''

    assert inspect(voice).is_well_formed()
    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'8 [
            d'8 ]
            e'8 [
            f'8 ]
        }
        '''
        )


def test_scoretools_Container_extend_03():
    r'''Extending container with empty list leaves container unchanged.
    '''

    voice = Voice("c'8 d'8")
    beam = Beam()
    attach(beam, voice[:])
    voice.extend([])

    r'''
    \new Voice {
        c'8 [
        d'8 ]
    }
    '''

    assert inspect(voice).is_well_formed()
    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'8 [
            d'8 ]
        }
        '''
        )


def test_scoretools_Container_extend_04():
    r'''Extending one container with empty second container leaves both containers unchanged.
    '''

    voice = Voice("c'8 d'8")
    beam = Beam()
    attach(beam, voice[:])
    voice.extend(Voice([]))

    r'''
    \new Voice {
        c'8 [
        d'8 ]
    }
    '''

    assert inspect(voice).is_well_formed()
    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'8 [
            d'8 ]
        }
        '''
        )


def test_scoretools_Container_extend_05():
    r'''Trying to extend container with noncomponent raises TypeError.
    '''

    voice = Voice("c'8 d'8")
    beam = Beam()
    attach(beam, voice[:])

    assert pytest.raises(Exception, 'voice.extend(7)')
    assert pytest.raises(Exception, "voice.extend('foo')")


def test_scoretools_Container_extend_06():
    r'''Trying to extend container with noncontainer raises exception.
    '''

    voice = Voice("c'8 d'8")
    beam = Beam()
    attach(beam, voice[:])

    assert pytest.raises(AttributeError,
        'voice.extend(Note(4, (1, 4)))')
    assert pytest.raises(AttributeError,
        'voice.extend(Chord([2, 3, 5], (1, 4)))')


def test_scoretools_Container_extend_07():
    r'''Extend container with partial and spanned contents of other container.
    '''

    voice = Voice("c'8 d'8")
    beam = Beam()
    attach(beam, voice[:])

    r'''
    \new Voice {
        c'8 [
        d'8 ]
    }
    '''

    u = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, u[:])

    r'''
    \new Voice {
        c'8 [
        d'8
        e'8
        f'8 ]
    }
    '''

    voice.extend(u[-2:])

    "Container voice is now ..."

    r'''
    \new Voice {
        c'8 [
        d'8 ]
        e'8
        f'8
    }
    '''

    assert inspect(voice).is_well_formed()
    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'8 [
            d'8 ]
            e'8
            f'8
        }
        '''
        )

    "Container u is now ..."

    r'''
    \new Voice {
        c'8 [
        d'8 ]
    }
    '''

    assert inspect(u).is_well_formed()
    assert systemtools.TestManager.compare(
        u,
        r'''
        \new Voice {
            c'8 [
            d'8 ]
        }
        '''
        )


def test_scoretools_Container_extend_08():
    r'''Extend container with partial and spanned contents of other container.
    Covered span comes with components from donor container.
    '''

    voice = Voice("c'8 d'8")
    beam = Beam()
    attach(beam, voice[:])

    r'''
    \new Voice {
        c'8 [
        d'8 ]
    }
    '''

    u = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, u[:])
    slur = Slur()
    attach(slur, u[-2:])

    r'''
    \new Voice {
        c'8 [
        d'8
        e'8 (
        f'8 ] )
    }
    '''

    voice.extend(u[-2:])

    "Container voice is now ..."

    r'''
    \new Voice {
        c'8 [
        d'8 ]
        e'8 (
        f'8 )
    }
    '''

    assert inspect(voice).is_well_formed()
    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'8 [
            d'8 ]
            e'8 (
            f'8 )
        }
        '''
        )

    "Container u is now ..."

    r'''
    \new Voice {
        c'8 [
        d'8 ]
    }
    '''

    assert inspect(u).is_well_formed()
    assert systemtools.TestManager.compare(
        u,
        r'''
        \new Voice {
            c'8 [
            d'8 ]
        }
        '''
        )


def test_scoretools_Container_extend_09():
    r'''Extend container with LilyPond input string.
    '''

    container = Container([])
    container.extend("c'4 ( d'4 e'4 f'4 )")

    r'''
    {
        c'4 (
        d'4
        e'4
        f'4 )
    }
    '''

    assert inspect(container).is_well_formed()
    assert systemtools.TestManager.compare(
        container,
        r'''
        {
            c'4 (
            d'4
            e'4
            f'4 )
        }
        '''
        )



# -*- encoding: utf-8 -*-
import numbers


def is_nonnegative_integer(expr):
    '''True when `expr` equals a nonnegative integer.

    ::

        >>> mathtools.is_nonnegative_integer(99)
        True

    ::

        >>> mathtools.is_nonnegative_integer(0)
        True

    Otherwise false:

    ::

        >>> mathtools.is_nonnegative_integer(-1)
        False

    Returns boolean.
    '''

    if isinstance(expr, numbers.Number):
        if expr == int(expr):
            if 0 <= expr:
                return True

    return False



# -*- encoding: utf-8 -*-
from abjad import *


def test_pitchtools_spell_numbered_interval_number_01():

    named_interval = pitchtools.spell_numbered_interval_number(1, 0)
    assert named_interval == pitchtools.NamedInterval('perfect', 1)

    named_interval = pitchtools.spell_numbered_interval_number(1, 1)
    assert named_interval == pitchtools.NamedInterval(
        'augmented', 1)

    named_interval = pitchtools.spell_numbered_interval_number(2, 0)
    assert named_interval == pitchtools.NamedInterval('diminished', 2)

    named_interval = pitchtools.spell_numbered_interval_number(2, 1)
    assert named_interval == pitchtools.NamedInterval('minor', 2)

    named_interval = pitchtools.spell_numbered_interval_number(2, 2)
    assert named_interval == pitchtools.NamedInterval('major', 2)

    named_interval = pitchtools.spell_numbered_interval_number(2, 3)
    assert named_interval == pitchtools.NamedInterval('augmented', 2)


# TODO: Refactor all diatonic stuff so unison is 0 instead of 1 #

def test_pitchtools_spell_numbered_interval_number_02():

    named_interval = pitchtools.spell_numbered_interval_number(1, 0)
    assert named_interval == pitchtools.NamedInterval('perfect', 1)

    named_interval = pitchtools.spell_numbered_interval_number(1, -1)
    assert named_interval == pitchtools.NamedInterval(
        'augmented', -1)

    named_interval = pitchtools.spell_numbered_interval_number(2, 0)
    # THE ASCENDING INTERVAL HERE IS WEIRD #
    assert named_interval == pitchtools.NamedInterval(
        'diminished', 2)

    named_interval = pitchtools.spell_numbered_interval_number(2, -1)
    assert named_interval == pitchtools.NamedInterval('minor', -2)

    named_interval = pitchtools.spell_numbered_interval_number(2, -2)
    assert named_interval == pitchtools.NamedInterval('major', -2)

    named_interval = pitchtools.spell_numbered_interval_number(2, -3)
    assert named_interval == pitchtools.NamedInterval(
        'augmented', -2)



# -*- encoding: utf-8 -*-


def make_big_centered_page_number_markup(text=None):
    r'''Make big centered page number markup:

    ::

        >>> markup = markuptools.make_big_centered_page_number_markup()

    ::

        >>> print format(markup, 'lilypond')
        \markup {
            \fill-line
                {
                    \bold
                        \fontsize
                            #3
                            \concat
                                {
                                    \on-the-fly
                                        #print-page-number-check-first
                                        \fromproperty
                                            #'page:page-number-string
                                }
                }
            }

    Returns markup.
    '''
    from abjad.tools import markuptools

    assert isinstance(text, (str, type(None)))

    if text is None:
        contents = r'''
        \fill-line {
        \bold \fontsize #3 \concat {
        \on-the-fly #print-page-number-check-first
        \fromproperty #'page:page-number-string } }'''
    else:
        contents = r'''
        \fill-line {
        \bold \fontsize #3 \concat {
        %s " " \char #x2014 " "
        \on-the-fly #print-page-number-check-first
        \fromproperty #'page:page-number-string } }''' % text

    markup = markuptools.Markup(contents)

    return markup



# -*- encoding: utf-8 -*-
from abjad import *


def test_markuptools_Markup_contents_01():

    markup = markuptools.Markup('foo')

    assert markup.contents == ('foo',)


def test_markuptools_Markup_contents_02():

    markup = markuptools.Markup(['foo', 'bar', 'baz'])

    assert markup.contents == ('foo', 'bar', 'baz')



import types


def detach(prototype, component_expression):
    r'''Detaches from `component_expression` all items matching `prototype`.

    Returns tuple of zero or more detached items.
    '''
    from abjad.tools import indicatortools
    from abjad.tools import scoretools
    from abjad.tools import spannertools
    from abjad.tools.agenttools.InspectionAgent import inspect

    spanners = []
    grace_containers = []
    inspector = inspect(component_expression)
    if isinstance(prototype, types.TypeType):
        if issubclass(prototype, spannertools.Spanner):
            spanners = inspector.get_spanners(prototype)
        elif issubclass(prototype, scoretools.GraceContainer):
            grace_containers = inspector.get_grace_containers(prototype)
        else:
            assert hasattr(component_expression, '_indicators')
            result = []
            for x in component_expression._indicators[:]:
                if isinstance(x, prototype):
                    component_expression._indicators.remove(x)
                    result.append(x)
                # indicator is a expression
                elif hasattr(x, 'indicator') and \
                    isinstance(x.indicator, prototype):
                    x._detach()
                    result.append(x.indicator)
            result = tuple(result)
            return result
    else:
        if isinstance(prototype, spannertools.Spanner):
            spanners = inspector.get_spanners(prototype)
        elif isinstance(prototype, scoretools.GraceContainer):
            grace_containers = inspector.get_grace_containers(prototype)
        else:
            assert hasattr(component_expression, '_indicators')
            result = []
            for x in component_expression._indicators[:]:
                if x == prototype:
                    component_expression._indicators.remove(x)
                    result.append(x)
                # indicator is a expression
                elif hasattr(x, 'indicator') and x.indicator == prototype:
                    x._detach()
                    result.append(x.indicator)
            result = tuple(result)
            return result
    items = []
    items.extend(spanners)
    items.extend(grace_containers)
    for item in items:
        item._detach()
    items = tuple(items)
    return items



# -*- encoding: utf-8 -*-
from abjad import *
import pytest


def test_lilypondparsertools_LilyPondParser__lilypondfile__HeaderBlock_01():

    string = r'''
    globalvariable = "This is a global variable."
    \header {
        globalvariable = "This overrides the global variable"
        localvariable = "and this is a local variable."
        title = \markup { \globalvariable \localvariable }
        something = #4
    }
    \score {
        \new Staff { c'4 ^ \markup { \globalvariable } }
    }
    '''

    result = parse(string)

    assert isinstance(result, lilypondfiletools.LilyPondFile)
    assert len(result) == 2

    assert systemtools.TestManager.compare(
        result[0],
        r'''
        \header {
            globalvariable = #"This overrides the global variable"
            localvariable = #"and this is a local variable."
            something = #4
            title = \markup {
                "This overrides the global variable"
                "and this is a local variable."
                }
        }
        '''
        )

    r'''
    \header {
        globalvariable = #"This overrides the global variable"
        localvariable = #"and this is a local variable."
        something = #4
        title = \markup {
            "This overrides the global variable"
            "and this is a local variable."
        }
    }
    '''

    assert systemtools.TestManager.compare(
        result[1],
        r'''
        \score {
            \new Staff {
                c'4 ^ \markup { This is a global variable. }
            }
        }
        '''
        )

    r'''
    \score {
        \new Staff {
            c'4 ^ \markup { This is a global variable. }
        }
    }
    '''



# -*- encoding: utf-8 -*-
from abjad.tools.stringtools.strip_diacritics_from_binary_string \
	import strip_diacritics_from_binary_string


def string_to_accent_free_snake_case(string):
    '''Change `string` to strict directory name:

    ::

        >>> stringtools.string_to_accent_free_snake_case('Déja vu')
        'deja_vu'

    Strip accents from accented characters.
    Change all punctuation (including spaces) to underscore.
    Set to lowercase.

    Returns string.
    '''

    assert isinstance(string, str)

    result = strip_diacritics_from_binary_string(string)
    result = result.replace(' ', '_')
    result = result.replace("'", '_')
    result = result.lower()

    return result



# -*- encoding: utf-8 -*-
from abjad.tools import datastructuretools
from abjad.tools.abctools.AbjadObject import AbjadObject


class InstrumentationSpecifier(AbjadObject):
    r'''Instrumentation specifier for an entire score.

    ::

        >>> flute = instrumenttools.Performer('Flute')
        >>> flute.instruments.append(instrumenttools.Flute())
        >>> flute.instruments.append(instrumenttools.AltoFlute())

    ::

        >>> guitar = instrumenttools.Performer('Guitar')
        >>> guitar.instruments.append(instrumenttools.Guitar())

    ::

        >>> instrumentation_specifier = \
        ...     instrumenttools.InstrumentationSpecifier([flute, guitar])

    ::

        >>> print format(instrumentation_specifier)
        instrumenttools.InstrumentationSpecifier(
            performers=instrumenttools.PerformerInventory(
                [
                    instrumenttools.Performer(
                        name='Flute',
                        instruments=instrumenttools.InstrumentInventory(
                            [
                                instrumenttools.Flute(
                                    instrument_name='flute',
                                    short_instrument_name='fl.',
                                    instrument_name_markup=markuptools.Markup(
                                        ('Flute',)
                                        ),
                                    short_instrument_name_markup=markuptools.Markup(
                                        ('Fl.',)
                                        ),
                                    allowable_clefs=indicatortools.ClefInventory(
                                        [
                                            indicatortools.Clef(
                                                'treble'
                                                ),
                                            ]
                                        ),
                                    pitch_range=pitchtools.PitchRange(
                                        '[C4, D7]'
                                        ),
                                    sounding_pitch_of_written_middle_c=pitchtools.NamedPitch("c'"),
                                    ),
                                instrumenttools.AltoFlute(
                                    instrument_name='alto flute',
                                    short_instrument_name='alt. fl.',
                                    instrument_name_markup=markuptools.Markup(
                                        ('Alto flute',)
                                        ),
                                    short_instrument_name_markup=markuptools.Markup(
                                        ('Alt. fl.',)
                                        ),
                                    allowable_clefs=indicatortools.ClefInventory(
                                        [
                                            indicatortools.Clef(
                                                'treble'
                                                ),
                                            ]
                                        ),
                                    pitch_range=pitchtools.PitchRange(
                                        '[G3, G6]'
                                        ),
                                    sounding_pitch_of_written_middle_c=pitchtools.NamedPitch('g'),
                                    ),
                                ]
                            ),
                        ),
                    instrumenttools.Performer(
                        name='Guitar',
                        instruments=instrumenttools.InstrumentInventory(
                            [
                                instrumenttools.Guitar(
                                    instrument_name='guitar',
                                    short_instrument_name='gt.',
                                    instrument_name_markup=markuptools.Markup(
                                        ('Guitar',)
                                        ),
                                    short_instrument_name_markup=markuptools.Markup(
                                        ('Gt.',)
                                        ),
                                    allowable_clefs=indicatortools.ClefInventory(
                                        [
                                            indicatortools.Clef(
                                                'treble'
                                                ),
                                            ]
                                        ),
                                    pitch_range=pitchtools.PitchRange(
                                        '[E2, E5]'
                                        ),
                                    sounding_pitch_of_written_middle_c=pitchtools.NamedPitch('c'),
                                    ),
                                ]
                            ),
                        ),
                    ]
                ),
            )

    Returns instrumentation specifier.
    '''

    def __init__(self, performers=None):
        from abjad.tools import instrumenttools
        self._performers = instrumenttools.PerformerInventory()
        self.performers = performers

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''True when `expr` is an instrumentation specifier with performers
        equal to those of instrumentation specifier. Otherwise false.

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            if self.performers == expr.performers:
                return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats instrumentation specifier.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    ### PUBLIC PROPERTIES ###

    @property
    def instrument_count(self):
        r'''Number of instruments in score.

        ::

            >>> instrumentation_specifier.instrument_count
            3

        Returns nonnegative integer.
        '''
        return len(self.instruments)

    @property
    def instruments(self):
        r'''List of instruments derived from performers.

        ::

            >>> instrumentation_specifier.instruments
            [Flute(), AltoFlute(), Guitar()]

        Returns list.
        '''
        instruments = []
        for performer in self.performers:
            instruments.extend(performer.instruments)
        return instruments

    @property
    def performer_count(self):
        r'''Number of performers in score.

        ::

            >>> instrumentation_specifier.performer_count
            2

        Returns nonnegative integer.
        '''
        return len(self.performers)

    @property
    def performer_name_string(self):
        r'''String of performer names.

        ::

            >>> instrumentation_specifier.performer_name_string
            'Flute, Guitar'

        Returns string.
        '''
        if self.performers:
            return ', '.join([performer.name 
                for performer in self.performers])
        else:
            return ''

    @property
    def performers(self):
        r'''Gets and sets list of performers in score.

        ::

            >>> print format(instrumentation_specifier.performers)
            instrumenttools.PerformerInventory(
                [
                    instrumenttools.Performer(
                        name='Flute',
                        instruments=instrumenttools.InstrumentInventory(
                            [
                                instrumenttools.Flute(
                                    instrument_name='flute',
                                    short_instrument_name='fl.',
                                    instrument_name_markup=markuptools.Markup(
                                        ('Flute',)
                                        ),
                                    short_instrument_name_markup=markuptools.Markup(
                                        ('Fl.',)
                                        ),
                                    allowable_clefs=indicatortools.ClefInventory(
                                        [
                                            indicatortools.Clef(
                                                'treble'
                                                ),
                                            ]
                                        ),
                                    pitch_range=pitchtools.PitchRange(
                                        '[C4, D7]'
                                        ),
                                    sounding_pitch_of_written_middle_c=pitchtools.NamedPitch("c'"),
                                    ),
                                instrumenttools.AltoFlute(
                                    instrument_name='alto flute',
                                    short_instrument_name='alt. fl.',
                                    instrument_name_markup=markuptools.Markup(
                                        ('Alto flute',)
                                        ),
                                    short_instrument_name_markup=markuptools.Markup(
                                        ('Alt. fl.',)
                                        ),
                                    allowable_clefs=indicatortools.ClefInventory(
                                        [
                                            indicatortools.Clef(
                                                'treble'
                                                ),
                                            ]
                                        ),
                                    pitch_range=pitchtools.PitchRange(
                                        '[G3, G6]'
                                        ),
                                    sounding_pitch_of_written_middle_c=pitchtools.NamedPitch('g'),
                                    ),
                                ]
                            ),
                        ),
                    instrumenttools.Performer(
                        name='Guitar',
                        instruments=instrumenttools.InstrumentInventory(
                            [
                                instrumenttools.Guitar(
                                    instrument_name='guitar',
                                    short_instrument_name='gt.',
                                    instrument_name_markup=markuptools.Markup(
                                        ('Guitar',)
                                        ),
                                    short_instrument_name_markup=markuptools.Markup(
                                        ('Gt.',)
                                        ),
                                    allowable_clefs=indicatortools.ClefInventory(
                                        [
                                            indicatortools.Clef(
                                                'treble'
                                                ),
                                            ]
                                        ),
                                    pitch_range=pitchtools.PitchRange(
                                        '[E2, E5]'
                                        ),
                                    sounding_pitch_of_written_middle_c=pitchtools.NamedPitch('c'),
                                    ),
                                ]
                            ),
                        ),
                    ]
                )

        Returns performer inventory.
        '''
        return self._performers

    @performers.setter
    def performers(self, performers):
        from abjad.tools import instrumenttools
        assert isinstance(performers, 
            (list, datastructuretools.TypedList, type(None)))
        if performers is None:
            self._performers[:] = []
        else:
            assert all(isinstance(x, instrumenttools.Performer) 
                for x in performers)
            self._performers[:] = list(performers[:])



# -*- encoding: utf-8 -*-
from abjad import *


def test_spannertools_HiddenStaffSpanner___init___01():
    r'''Initialize empty hidden staff spanner.
    '''

    spanner = spannertools.HiddenStaffSpanner()
    assert isinstance(spanner, spannertools.HiddenStaffSpanner)


def test_spannertools_HiddenStaffSpanner___init___02():
    r'''Hide staff around one measure.
    '''

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")
    scoretools.set_always_format_time_signature_of_measures_in_expr(staff)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8
            d'8
        }
        {
            \time 2/8
            e'8
            f'8
        }
        {
            \time 2/8
            g'8
            a'8
        }
    }
    '''

    spanner = spannertools.HiddenStaffSpanner()
    attach(spanner, staff[1])

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                \time 2/8
                \stopStaff
                e'8
                f'8
                \startStaff
            }
            {
                \time 2/8
                g'8
                a'8
            }
        }
        '''
        )

    assert inspect(staff).is_well_formed()


def test_spannertools_HiddenStaffSpanner___init___03():
    r'''Hide staff around one leaf.
    '''

    note = Note(0, (1, 8))
    spanner = spannertools.HiddenStaffSpanner()
    attach(spanner, note)

    assert systemtools.TestManager.compare(
        note,
        r'''
        \stopStaff
        c'8
        \startStaff
        '''
        )

    assert inspect(note).is_well_formed()



# -*- encoding: utf-8 -*-
from abjad.tools.lilypondfiletools.NonattributedBlock \
	import NonattributedBlock


class BookpartBlock(NonattributedBlock):
    r'''Abjad model of LilyPond input file bookpart block:

    ::

        >>> bookpart_block = lilypondfiletools.BookpartBlock()

    ::

        >>> bookpart_block
        BookpartBlock()

    ..  doctest::

        >>> print format(bookpart_block)
        \bookpart {}

    Returns bookpart block.
    '''

    def __init__(self):
        NonattributedBlock.__init__(self)
        self._escaped_name = r'\bookpart'
        self._is_formatted_when_empty = True



# -*- encoding: utf-8 -*-
from abjad import *


def test_spannertools_make_colored_text_spanner_with_nibs_01():
    r'''Bracket defaults to solid red line with left and right nibs
    and with no nibs at left and right broken edges.
    '''

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")
    spanner = spannertools.make_colored_text_spanner_with_nibs()
    attach(spanner, staff[1])
    scoretools.set_always_format_time_signature_of_measures_in_expr(staff)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                \time 2/8
                \override TextSpanner #'bound-details #'left #'text = \markup {
                    \draw-line
                        #'(0 . -1)
                    }
                \override TextSpanner #'bound-details #'left-broken #'text = ##f
                \override TextSpanner #'bound-details #'right #'text = \markup {
                    \draw-line
                        #'(0 . -1)
                    }
                \override TextSpanner #'bound-details #'right-broken #'text = ##f
                \override TextSpanner #'color = #red
                \override TextSpanner #'dash-fraction = #1
                \override TextSpanner #'staff-padding = #2
                \override TextSpanner #'thickness = #1.5
                e'8 \startTextSpan
                f'8 \stopTextSpan
                \revert TextSpanner #'bound-details
                \revert TextSpanner #'color
                \revert TextSpanner #'dash-fraction
                \revert TextSpanner #'staff-padding
                \revert TextSpanner #'thickness
            }
            {
                \time 2/8
                g'8
                a'8
            }
        }
        ''',
        )



# -*- encoding: utf-8 -*-
from abjad.tools import scoretools
from abjad.tools.topleveltools import override


def color_leaf(leaf, color):
    r'''Color note:

    ::

        >>> note = Note("c'4")

    ::

        >>> labeltools.color_leaf(note, 'red')
        Note("c'4")

    ..  doctest::

        >>> print format(note)
        \once \override Accidental #'color = #red
        \once \override Beam #'color = #red
        \once \override Dots #'color = #red
        \once \override NoteHead #'color = #red
        \once \override Stem #'color = #red
        c'4

    ::

        >>> show(note) # doctest: +SKIP

    Color rest:

    ::

        >>> rest = Rest('r4')

    ::

        >>> labeltools.color_leaf(rest, 'red')
        Rest('r4')

    ..  doctest::

        >>> print format(rest)
        \once \override Dots #'color = #red
        \once \override Rest #'color = #red
        r4

    ::

        >>> show(rest) # doctest: +SKIP

    Color chord:

    ::

        >>> chord = Chord("<c' e' bf'>4")

    ::

        >>> labeltools.color_leaf(chord, 'red')
        Chord("<c' e' bf'>4")

    ..  doctest::

        >>> print format(chord)
        \once \override Accidental #'color = #red
        \once \override Beam #'color = #red
        \once \override Dots #'color = #red
        \once \override NoteHead #'color = #red
        \once \override Stem #'color = #red
        <c' e' bf'>4

    ::

        >>> show(chord) # doctest: +SKIP

    Return `leaf`.
    '''

    # color leaf
    if isinstance(leaf, (scoretools.Note, scoretools.Chord)):
        override(leaf).accidental.color = color
        override(leaf).beam.color = color
        override(leaf).dots.color = color
        override(leaf).note_head.color = color
        override(leaf).stem.color = color
    elif isinstance(leaf, scoretools.Rest):
        override(leaf).dots.color = color
        override(leaf).rest.color = color

    # return leaf
    return leaf



# -*- encoding: utf-8 -*-
from abjad import *


def test_templatetools_StringQuartetScoreTemplate___call___01():

    template = templatetools.StringQuartetScoreTemplate()
    score = template()

    r'''
    \context Score = "String Quartet Score" <<
        \context StaffGroup = "String Quartet Staff Group" <<
            \context Staff = "First Violin Staff" {
                \clef "treble"
                \set Staff.instrumentName = \markup { Violin }
                \set Staff.shortInstrumentName = \markup { Vn. }
                \context Voice = "First Violin Voice" {
                }
            }
            \context Staff = "Second Violin Staff" {
                \clef "treble"
                \set Staff.instrumentName = \markup { Violin }
                \set Staff.shortInstrumentName = \markup { Vn. }
                \context Voice = "Second Violin Voice" {
                }
            }
            \context Staff = "Viola Staff" {
                \clef "alto"
                \set Staff.instrumentName = \markup { Viola }
                \set Staff.shortInstrumentName = \markup { Va. }
                \context Voice = "Viola Voice" {
                }
            }
            \context Staff = "Cello Staff" {
                \clef "bass"
                \set Staff.instrumentName = \markup { Cello }
                \set Staff.shortInstrumentName = \markup { Vc. }
                \context Voice = "Cello Voice" {
                }
            }
        >>
    >>
    '''

    assert systemtools.TestManager.compare(
        score,
        r'''
        \context Score = "String Quartet Score" <<
            \context StaffGroup = "String Quartet Staff Group" <<
                \context Staff = "First Violin Staff" {
                    \clef "treble"
                    \set Staff.instrumentName = \markup { Violin }
                    \set Staff.shortInstrumentName = \markup { Vn. }
                    \context Voice = "First Violin Voice" {
                    }
                }
                \context Staff = "Second Violin Staff" {
                    \clef "treble"
                    \set Staff.instrumentName = \markup { Violin }
                    \set Staff.shortInstrumentName = \markup { Vn. }
                    \context Voice = "Second Violin Voice" {
                    }
                }
                \context Staff = "Viola Staff" {
                    \clef "alto"
                    \set Staff.instrumentName = \markup { Viola }
                    \set Staff.shortInstrumentName = \markup { Va. }
                    \context Voice = "Viola Voice" {
                    }
                }
                \context Staff = "Cello Staff" {
                    \clef "bass"
                    \set Staff.instrumentName = \markup { Cello }
                    \set Staff.shortInstrumentName = \markup { Vc. }
                    \context Voice = "Cello Voice" {
                    }
                }
            >>
        >>
        '''
        )



# -*- encoding: utf-8 -*-
import collections
import copy


# TODO: Reimplement to work on Abjad PitchSet, Note and Chord objects only.
# TODO: Reimplement to work with diatonic transposition. #
def list_octave_transpositions_of_pitch_carrier_within_pitch_range(
    pitch_carrier, pitch_range):
    r"""List octave transpositions of `pitch_carrier` in `pitch_range`:

    ::

        >>> chord = Chord("<c' d' e'>4")
        >>> pitch_range = pitchtools.PitchRange(0, 48)

    ::

        >>> result = pitchtools.list_octave_transpositions_of_pitch_carrier_within_pitch_range(
        ...     chord, pitch_range)

    ::

        >>> for chord in result:
        ...     chord
        ...
        Chord("<c' d' e'>4")
        Chord("<c'' d'' e''>4")
        Chord("<c''' d''' e'''>4")
        Chord("<c'''' d'''' e''''>4")

    Returns list of newly created `pitch_carrier` objects.
    """
    from abjad.tools import scoretools
    from abjad.tools import pitchtools

    if not isinstance(pitch_range, pitchtools.PitchRange):
        message = 'must be pitch range.'
        raise TypeError(message)

    if isinstance(pitch_carrier, collections.Iterable):
        if all(isinstance(x, (int, long, float)) for x in pitch_carrier):
            return _pitch_number_list_octave_transpositions(
                pitch_carrier, pitch_range)

    if not isinstance(pitch_carrier, (scoretools.Chord, pitchtools.PitchSet)):
        message = 'must be chord or pitch-set: {!r}'
        message = message.format(pitch_carrier)
        raise TypeError(message)

    result = []

    interval = pitchtools.NumberedInterval(-12)
    while True:
        pitch_carrier_copy = copy.copy(pitch_carrier)
        candidate = pitchtools.transpose_pitch_carrier_by_interval(
            pitch_carrier_copy, interval)
        if candidate in pitch_range:
            result.append(candidate)
            interval -= pitchtools.NumberedInterval(12)
        else:
            break

    result.reverse()

    interval = pitchtools.NumberedInterval(0)
    while True:
        pitch_carrier_copy = copy.copy(pitch_carrier)
        candidate = pitchtools.transpose_pitch_carrier_by_interval(
            pitch_carrier_copy, interval)
        if candidate in pitch_range:
            result.append(candidate)
            interval += pitchtools.NumberedInterval(12)
        else:
            break

    return result


# TODO: make public
def _pitch_number_list_octave_transpositions(pitch_number_list, pitch_range):
    result = []
    ps = set(pitch_number_list)
    start_pitch_number = pitch_range.start_pitch.pitch_number
    stop_pitch_number = pitch_range.stop_pitch.pitch_number
    R = set(range(start_pitch_number, stop_pitch_number + 1))
    while ps.issubset(R):
        next_pitch_number = list(ps)
        next_pitch_number.sort()
        result.extend([next_pitch_number])
        ps = set([p + 12 for p in ps])

    ps = set([p - 12 for p in pitch_number_list])
    while ps.issubset(R):
        next_pitch_number = list(ps)
        next_pitch_number.sort()
        result.extend([next_pitch_number])
        ps = set([p - 12 for p in ps])

    result.sort()
    return result



# -*- encoding: utf-8 -*-


def instantiate_pitch_and_interval_test_collection():
    r'''Instantiate pitch and interval test collection:

    ::

        >>> for x in pitchtools.instantiate_pitch_and_interval_test_collection(): x
        ...
        NumberedInversionEquivalentIntervalClass(1)
        NamedInversionEquivalentIntervalClass('+M2')
        NumberedInterval(1)
        NumberedIntervalClass(1)
        NamedInterval('+M2')
        NamedIntervalClass('+M2')
        NamedPitch('c')
        NamedPitchClass('c')
        NumberedPitch(1)
        NumberedPitchClass(1)

    Use to test pitch and interval interface consistency.

    Returns list.
    '''
    from abjad.tools import pitchtools

    result = []
    result.append(pitchtools.NumberedInversionEquivalentIntervalClass(1))
    result.append(pitchtools.NamedInversionEquivalentIntervalClass('M2'))
    result.append(pitchtools.NumberedInterval(1))
    result.append(pitchtools.NumberedIntervalClass(1))
    result.append(pitchtools.NamedInterval('M2'))
    result.append(pitchtools.NamedIntervalClass('M2'))
    result.append(pitchtools.NamedPitch('c'))
    result.append(pitchtools.NamedPitchClass('c'))
    result.append(pitchtools.NumberedPitch(1))
    result.append(pitchtools.NumberedPitchClass(1))
    return result



# -*- encoding: utf-8 -*-
from abjad import *


def test_rhythmmakertools_NoteRhythmMaker___eq___01():

    maker_1 = rhythmmakertools.NoteRhythmMaker()
    maker_2 = rhythmmakertools.NoteRhythmMaker()

    assert maker_1 == maker_2
    assert maker_2 == maker_1
    assert not maker_1 == 'foo'



# -*- encoding: utf-8 -*-
from abjad import *


def test_pitchtools_named_pitch_and_clef_to_staff_position_number_01():

    clef = Clef('treble')

    pitch = NamedPitch('c', 4)
    number = pitchtools.named_pitch_and_clef_to_staff_position_number(pitch, clef)
    assert number == -6

    pitch = NamedPitch('d', 4)
    number = pitchtools.named_pitch_and_clef_to_staff_position_number(pitch, clef)
    assert number == -5

    pitch = NamedPitch('e', 4)
    number = pitchtools.named_pitch_and_clef_to_staff_position_number(pitch, clef)
    assert number == -4


def test_pitchtools_named_pitch_and_clef_to_staff_position_number_02():

    clef = Clef('alto')

    pitch = NamedPitch('c', 4)
    number = pitchtools.named_pitch_and_clef_to_staff_position_number(pitch, clef)
    assert number == 0

    pitch = NamedPitch('d', 4)
    number = pitchtools.named_pitch_and_clef_to_staff_position_number(pitch, clef)
    assert number == 1

    pitch = NamedPitch('e', 4)
    number = pitchtools.named_pitch_and_clef_to_staff_position_number(pitch, clef)
    assert number == 2



# -*- encoding: utf-8 -*-
from abjad.tools.mathtools.NonreducedRatio import NonreducedRatio


class Ratio(NonreducedRatio):
    '''Ratio of one or more nonzero integers.

    Initializes from one or more nonzero integers:

    ::

        >>> mathtools.Ratio(2, 4, 2)
        Ratio(1, 2, 1)

    Initializes from a tuple or list:

    ::

        >>> ratio = mathtools.Ratio((2, 4, 2))
        >>> ratio
        Ratio(1, 2, 1)

    Uses a tuple to return ratio integers.

        >>> tuple(ratio)
        (1, 2, 1)

    Ratios are immutable.
    '''

    ### CONSTRUCTOR ###

    def __new__(cls, *args):
        from abjad.tools import sequencetools
        if len(args) == 1 and isinstance(args[0], (list, tuple)):
            args = args[0]
        elif len(args) == 0:
            args = (1, 1)
        assert args, repr(args)
        assert all(x != 0 for x in args), repr(args)
        args = \
            sequencetools.divide_sequence_elements_by_greatest_common_divisor(
            args)
        self = NonreducedRatio.__new__(cls, args)
        return self

    ### SPECIAL METHODS ###

    def __str__(self):
        r'''String representation of ratio.

        Returns string.
        '''
        terms = (str(x) for x in self)
        return ':'.join(terms)



# -*- encoding: utf-8 -*-
import importlib
import inspect
import os
import shutil
import subprocess
from abjad.tools import sequencetools
from abjad.tools import documentationtools
from abjad.tools.abctools.AbjadObject import AbjadObject


class AbjadBookProcessor(AbjadObject):
    r'''Abjad book processor.
    '''

    ### INITIALIZER ###

    def __init__(
        self, 
        directory=None, 
        lines=None, 
        output_format=None, 
        skip_rendering=False,
        image_prefix='image', 
        verbose=False,
        ):
        from abjad.tools import abjadbooktools
        directory = directory = '.'
        lines = lines or []
        output_format = output_format or abjadbooktools.HTMLOutputFormat()
        assert isinstance(output_format, abjadbooktools.OutputFormat)
        self._current_code_line = 0
        self._total_code_lines = 0
        self._directory = os.path.abspath(directory)
        self._image_prefix = image_prefix
        self._lines = tuple(lines)
        self._output_format = output_format
        self._skip_rendering = bool(skip_rendering)
        self._verbose = bool(verbose)

    ### SPECIAL METHOD ###

    def __call__(self, verbose=True):
        r'''Calls Abjad book processor.
        '''

        # Verify input, and extract code blocks
        code_blocks = self._extract_code_blocks(self.lines)
        for code_block in code_blocks:
            self._total_code_lines += len(code_block.lines)

        # Create a temporary directory, and step into it.
        tmp_directory = self._setup_tmp_directory(self.directory)
        os.chdir(tmp_directory)

        # Process code blocks and render images inside the temporary directory
        pipe = self._setup_pipe()
        image_count = self._process_code_blocks(
            pipe, code_blocks, tmp_directory, self.image_prefix)
        ly_file_names = self._extract_ly_file_names(code_blocks)
        self._cleanup_pipe(pipe)
        if not self.skip_rendering:
            self._render_ly_files(ly_file_names, self.output_format, verbose)

        # Interleave newly reformatted code with the old, and return.
        if code_blocks:
            result = self._interleave_source_with_code_blocks(
                tmp_directory, self.lines, code_blocks, self.output_format)
        else:
            result = '\n'.join(self.lines)

        # Step out of the tmp directory, back to the original, and cleanup.
        os.chdir(self.directory)
        self._cleanup_image_files(
            self.directory, tmp_directory, image_count, self.image_prefix,
            self.output_format.image_format)
        self._cleanup_tmp_directory(tmp_directory)

        return result

    ### PUBLIC PROPERTIES ###

    @property
    def directory(self):
        r'''Directory.
        '''
        return self._directory

    @property
    def image_prefix(self):
        r'''Image prefix.
        '''
        return self._image_prefix

    @property
    def lines(self):
        r'''Lines.
        '''
        return self._lines

    @property
    def output_format(self):
        r'''Output format.
        '''
        return self._output_format

    @property
    def skip_rendering(self):
        r'''Skip rendering.
        '''
        return self._skip_rendering

    @property
    def verbose(self):
        r'''Verbose.
        '''
        return self._verbose

    ### PRIVATE METHODS ###

    def _cleanup_image_files(self, directory, tmp_directory,
        image_count, image_prefix, image_format):
        #print 'CLEANUP IMAGE FILES'
        image_directory = os.path.join(directory, 'images')
        if not os.path.isdir(image_directory):
            os.mkdir(image_directory)
        # remove old images
        #for x in os.listdir(image_directory):
        #    if x.startswith('{}-'.format(image_prefix)) and \
        #        x.endswith(image_format):
        #        # this should handle both 'index-1.png' and 'index-1-page3.png'
        #        name = os.path.splitext(x)[0]
        #        number = int(name.split('-')[1])
        #        if image_count < number:
        #            os.remove(os.path.join(image_directory, x))
        for x in os.listdir(tmp_directory):
            if x.endswith('.png'):
                source = os.path.join(tmp_directory, x)
                target = os.path.join(image_directory, x)
                if not os.path.exists(target) or \
                    not documentationtools.compare_images(source, target):
                    os.rename(source, target)
                    if self.verbose:
                        print '\tMoving {}.'.format(x)
                else:
                    if self.verbose:
                        print '\tKeeping old {}.'.format(x)
            elif x.endswith('.pdf'):
                source = os.path.join(tmp_directory, x)
                target = os.path.join(image_directory, x)
                os.rename(source, target)
                if self.verbose:
                    print '\tMoving {}.'.format(x)

    def _cleanup_pipe(self, pipe):
        #print 'CLEANUP PIPE'
        pipe.write('quit()\n')
        pipe.close()

    def _cleanup_tmp_directory(self, tmp_directory):
        #print 'CLEANUP TMP DIRECTORY'
        shutil.rmtree(tmp_directory)

    def _extract_code_blocks(self, lines):
        from abjad.tools import abjadbooktools
        #print 'EXTRACT CODE BLOCKS'
        blocks = []
        block = []
        starting_line_number = 0
        in_block = False
        for i, line in enumerate(lines):
            if line.startswith('<abjad>'):
                if in_block:
                    message = 'extra opening tag at line {}.'.format(i)
                    raise Exception(message)
                else:
                    in_block = True
                    block = [line]
                    starting_line_number = i

            elif line.startswith('</abjad>'):
                if in_block:
                    in_block = False
                    hide = 'hide=true' in block[0]
                    strip_prompt = 'strip_prompt=true' in block[0]
                    stopping_line_number = i
                    code_block = abjadbooktools.CodeBlock(
                        block[1:],
                        starting_line_number,
                        stopping_line_number,
                        hide=hide,
                        strip_prompt=strip_prompt
                        )
                    blocks.append(code_block)
                else:
                    message = 'extra closing tag at line {}'.format(i)
                    raise Exception(message)

            elif in_block:
                block.append(line)

            elif line.startswith('<abjadextract '):
                block = []
                starting_line_number = stopping_line_number = i
                hide = 'hide=true' in line
                strip_prompt = 'strip_prompt=true' in line
                code_address = line.partition(
                    '<abjadextract ')[-1].partition(' \>')[0]
                module_name, dot, attr_name = code_address.rpartition('.')
                module = importlib.import_module(module_name)
                attr = getattr(module, attr_name)
                code_lines = inspect.getsource(attr).splitlines()
                code_block = abjadbooktools.CodeBlock(
                    code_lines,
                    starting_line_number,
                    stopping_line_number,
                    hide=hide,
                    strip_prompt=strip_prompt
                    )
                blocks.append(code_block)

        if in_block:
            message = 'unterminated tag at EOF.'
            raise Exception(message)

        return tuple(blocks)

    def _extract_ly_file_names(self, code_blocks):
        #print 'EXTRACT LY FILENAMES'
        file_names = []
        for code_block in code_blocks:
            for result in code_block.processed_results:
                if isinstance(result, dict):
                    file_names.append(result['file_name'])
        return file_names

    def _interleave_source_with_code_blocks(
        self,
        tmp_directory,
        lines,
        code_blocks,
        output_format,
        ):

        #print 'INTERLEAVE SOURCE WITH CODE BLOCKS'
        image_file_names = [x for x in os.listdir(tmp_directory)
            if (x.endswith(output_format.image_format) and
               x.startswith(self.image_prefix))]

        image_dict = {}
        for image_filename in image_file_names:
            suffix = os.path.splitext(image_filename.partition('-')[2])[0]
            index, part, page = suffix.partition('-')
            index = int(index)
            if page:
                page = int(page.strip('page'))
            else:
                page = 0
            if index not in image_dict:
                image_dict[index] = {}
            image_dict[index][page] = image_filename

        interleaved = []
        interleaved.append(
            '\n'.join(lines[:code_blocks[0].starting_line_number]))
        for pair in sequencetools.iterate_sequence_pairwise_strict(
            code_blocks):
            first_block, second_block = pair
            interleaved.extend(output_format(first_block, image_dict))
            interleaved.append('\n'.join(lines[
                first_block.ending_line_number+1:
                second_block.starting_line_number]))

        interleaved.extend(output_format(code_blocks[-1], image_dict))
        interleaved.append('\n'.join(
            lines[code_blocks[-1].ending_line_number+1:]))
        return '\n'.join(interleaved)

    def _process_code_blocks(
        self,
        pipe,
        code_blocks,
        directory,
        image_prefix,
        ):
        #print 'PROCESS CODE BLOCKS'
        image_count = 0
        for i, code_block in enumerate(code_blocks):
            #print '\tCODE BLOCK', i
            image_count = code_block(
                self,
                pipe,
                image_count,
                directory,
                image_prefix,
                verbose=self.verbose,
                )
        return image_count

    def _render_ly_files(self, file_names, output_format, verbose):
        #print 'RENDER LY FILES'
        for file_name in file_names:
            if self.verbose:
                print '\tRendering {}.ly ...'.format(file_name)
            try:
                if output_format.image_format == 'pdf':
                    command = 'lilypond {}.ly'.format(file_name)
                    if self.verbose:
                        print '\t\t{}'.format(command)
                    self._run_command(command, verbose)
                    command = 'pdfcrop {}.pdf {}.pdf'.format(
                        file_name, file_name)
                    if self.verbose:
                        print '\t\t{}'.format(command)
                    self._run_command(command, verbose)
                elif output_format.image_format == 'png':
                    command = 'lilypond --png -dresolution=300 {}.ly'.format(
                        file_name)
                    if self.verbose:
                        print '\t\t{}'.format(command)
                    assert os.path.exists('{}.ly'.format(file_name))
                    self._run_command(command, verbose)
                    for file in os.listdir('.'):
                        if file.startswith(file_name) and file.endswith('.png'):
                            command = '\
                                convert {} -trim -resample 40%% {}'.format(
                                    file, file)
                            if self.verbose:
                                print '\t\t{}'.format(command)
                            self._run_command(command, verbose)
            except AssertionError, e:
                print e

    def _run_command(self, command, verbose):
        if verbose:
            subprocess.call(command, shell=True)
        else:
            subprocess.call(
                command,
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                )

    def _setup_pipe(self):
        #print 'SETUP PIPE'
        pipe = documentationtools.Pipe()
        pipe.read_wait()
        pipe.write('from abjad import *\n')
        pipe.read_wait()
        return pipe

    def _setup_tmp_directory(self, directory):
        #print 'SETUP TMP DIRECTORY'
        import tempfile
        tmp_directory = os.path.abspath(tempfile.mkdtemp(dir=directory))
        return tmp_directory

    ### PUBLIC METHODS ###

    def update_status(self, line):
        r'''Updates status.

        Returns none.
        '''
        self._current_code_line += 1
        percentage = float(self._current_code_line) / self._total_code_lines
        message = '[{:4.0%}] {}'.format(percentage, line)
        if self.verbose:
            print message



# -*- encoding: utf-8 -*-
import abc
import copy
from abjad.tools import datastructuretools
from abjad.tools import durationtools
from abjad.tools import scoretools
from abjad.tools import sequencetools
from abjad.tools import scoretools
from abjad.tools.abctools.AbjadObject import AbjadObject


class RhythmMaker(AbjadObject):
    '''Rhythm maker abstract base class.
    '''

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(
        self,
        forbidden_written_duration=None,
        beam_each_cell=True,
        beam_cells_together=False,
        ):
        self.forbidden_written_duration = forbidden_written_duration
        self.beam_each_cell = beam_each_cell
        self.beam_cells_together = beam_cells_together

    ### SPECIAL METHODS ###

    def __call__(self, divisions, seeds=None):
        r'''Casts `divisions` into duration pairs.
        Reduces numerator and denominator relative to each other.

        Changes none `seeds` into empty list.

        Returns duration pairs and seed list.
        '''
        duration_pairs = [durationtools.Duration(x).pair for x in divisions]
        seeds = self._none_to_new_list(seeds)
        return duration_pairs, seeds

    def __eq__(self, expr):
        r'''True when `expr` is same type
        with the equal public nonhelper properties.
        Otherwise false.

        Returns boolean.
        '''
        from abjad.tools import systemtools
        if isinstance(expr, type(self)):
            manager = systemtools.StorageFormatManager
            if manager.get_positional_argument_values(self) == \
                manager.get_positional_argument_values(expr):
                nonhelper_keyword_argument_names = [
                    x for x in manager.get_keyword_argument_names(self)
                    if 'helper' not in x]
                for nonhelper_keyword_argument_name in \
                    nonhelper_keyword_argument_names:
                    if not getattr(self, nonhelper_keyword_argument_name) == \
                        getattr(expr, nonhelper_keyword_argument_name):
                        return False
                return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats rhythm-maker.

        Set `format_specification` to `''` or `'storage'`.

        Defaults `format_specification=None` to
        `format_specification='storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    def __getstate__(self):
        r'''Gets object state.
        '''
        return vars(self)

    ### PRIVATE METHODS ###

    # TODO: make static
    def _all_are_tuplets_or_all_are_leaf_lists(self, expr):
        if all(isinstance(x, scoretools.Tuplet) for x in expr):
            return True
        elif all(self._is_leaf_list(x) for x in expr):
            return True
        else:
            return False

    @staticmethod
    def _is_leaf_list(expr):
        return all(isinstance(x, scoretools.Leaf) for x in expr)

    def _make_secondary_duration_pairs(
        self, duration_pairs, secondary_divisions):
        if not secondary_divisions:
            return duration_pairs[:]
        numerators = [duration_pair.numerator
            for duration_pair in duration_pairs]
        secondary_numerators = sequencetools.split_sequence_by_weights(
            numerators, secondary_divisions, cyclic=True, overhang=True)
        secondary_numerators = \
            sequencetools.flatten_sequence(secondary_numerators)
        denominator = duration_pairs[0].denominator
        secondary_duration_pairs = \
            [(n, denominator) for n in secondary_numerators]
        return secondary_duration_pairs

    def _make_tuplets(self, duration_pairs, leaf_lists):
        assert len(duration_pairs) == len(leaf_lists)
        tuplets = []
        for duration_pair, leaf_list in zip(duration_pairs, leaf_lists):
            tuplet = scoretools.FixedDurationTuplet(duration_pair, leaf_list)
            tuplets.append(tuplet)
        return tuplets

    def _none_to_new_list(self, expr):
        if expr is None:
            return []
        return expr

    def _none_to_trivial_helper(self, expr):
        if expr is None:
            return self._trivial_helper
        return expr

    def _scale_talee(self, duration_pairs, talea_denominator, talee):
        dummy_duration_pair = (1, talea_denominator)
        duration_pairs.append(dummy_duration_pair)
        Duration = durationtools.Duration
        duration_pairs = \
            Duration.durations_to_nonreduced_fractions(
            duration_pairs)
        dummy_duration_pair = duration_pairs.pop()
        lcd = dummy_duration_pair.denominator
        multiplier = lcd / talea_denominator
        scaled_talee = []
        for talea in talee:
            talea = datastructuretools.CyclicTuple([multiplier * x for x in talea])
            scaled_talee.append(talea)
        result = [duration_pairs, lcd]
        result.extend(scaled_talee)
        return tuple(result)

    def _sequence_to_ellipsized_string(self, sequence):
        if not sequence:
            return '[]'
        if len(sequence) <= 4:
            result = ', '.join([str(x) for x in sequence])
        else:
            result = ', '.join([str(x) for x in sequence[:4]])
            result += ', ...'
        result = '[$%s$]' % result
        return result

    def _trivial_helper(self, talea, seeds):
        if isinstance(seeds, int) and len(talea):
            return sequencetools.rotate_sequence(talea, seeds)
        return talea

    ### PUBLIC METHODS ###

    def __makenew__(self, *args, **kwargs):
        r'''Creates new rhythm-maker with `kwargs`.

        ::

            >>> maker = rhythmmakertools.NoteRhythmMaker()

        ::

            >>> divisions = [(5, 16), (3, 8)]
            >>> new_maker = new(maker, decrease_durations_monotonically=False)
            >>> leaf_lists = new_maker(divisions)
            >>> leaves = sequencetools.flatten_sequence(leaf_lists)

        ::

            >>> staff = Staff(
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions))
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, leaves)

        ..  doctest::

            >>> print format(staff)
            \new Staff {
                {
                    \time 5/16
                    c'16 ~
                    c'4
                }
                {
                    \time 3/8
                    c'4.
                }
            }

        Returns new rhythm-maker.
        '''
        new_maker = copy.deepcopy(self)
        for key, value in kwargs.iteritems():
            try:
                setattr(new_maker, key, value)
            except AttributeError:
                setattr(new_maker, '_' + key, value)
        return new_maker

    def reverse(self):
        r'''Reverses rhythm-maker.

        .. note:: method is provisional.

        Defined equal to exact copy of rhythm-maker.

        This is the fallback for child classes.

        Directed rhythm-maker child classes should override this method.

        Returns newly constructed rhythm-maker.
        '''
        new = copy.deepcopy(self)
        return new



# -*- encoding: utf-8 -*-
from abjad.tools.abctools.AbjadObject import AbjadObject
from abjad.tools.pitchtools.NamedPitch import NamedPitch
from abjad.tools.pitchtools.NumberedPitch import NumberedPitch
from abjad.tools.pitchtools.PitchRange import PitchRange


class OctaveTranspositionMappingComponent(AbjadObject):
    '''An octave transposition mapping component.

    ::

        >>> mc = pitchtools.OctaveTranspositionMappingComponent('[A0, C8]', 15)
        >>> mc
        OctaveTranspositionMappingComponent('[A0, C8]', 15)

    Initializes from input parameters separately, from a pair, from
    a string or from another mapping component.

    Models
    ``pitchtools.transpose_pitch_number_by_octave_transposition_mapping``
    input part. (See the docs for that function.)

    Octave transposition mapping components are mutable.
    '''

    ### CLASS VARIABLES ###

    _default_positional_input_arguments = (
        repr('[A0, C8]'),
        15,
        )

    ### INITIALIZER ###

    def __init__(self, *args):
        if len(args) == 1 and isinstance(args[0], tuple):
            source_pitch_range, target_octave_start_pitch = args[0]
        elif len(args) == 1 and isinstance(args[0], str):
            assert ' => ' in args[0]
            source_pitch_range, target_octave_start_pitch = \
                args[0].split(' => ')
            try:
                target_octave_start_pitch = eval(target_octave_start_pitch)
            except NameError:
                target_octave_start_pitch = NamedPitch(
                    target_octave_start_pitch)
                target_octave_start_pitch = \
                    target_octave_start_pitch.pitch_number
        elif len(args) == 1 and isinstance(args[0], type(self)):
            source_pitch_range = args[0].source_pitch_range
            target_octave_start_pitch = args[0].target_octave_start_pitch
        elif len(args) == 2:
            source_pitch_range, target_octave_start_pitch = args
        else:
            raise ValueError(repr(args))
        self.source_pitch_range = source_pitch_range
        self.target_octave_start_pitch = target_octave_start_pitch

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''True when `expr` is a an octave transposition mapping component
        with source pitch range and target octave start pitch equal to those of
        this octave transposition mapping component. Otherwise false.

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            if self.source_pitch_range == expr.source_pitch_range:
                if self.target_octave_start_pitch == \
                    expr.target_octave_start_pitch:
                    return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats mapping component.

        Set `format_specification` to `''`, `'lilypond'` or `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    def __ne__(self, expr):
        r'''True when octave transposition mapping component does not equal 
        `expr`. Otherwise false.

        Returns boolean.
        '''
        return not self == expr

    ### PRIVATE PROPERTIES ###

    @property
    def _input_argument_token(self):
        return '({!r}, {:d})'.format(
            self.source_pitch_range.one_line_named_pitch_repr,
            self.target_octave_start_pitch)

    @property
    def _keyword_argument_names(self):
        return ()

    @property
    def _list_format(self):
        return ((
            self.source_pitch_range.start_pitch.pitch_number,
            self.source_pitch_range.stop_pitch.pitch_number),
            self.target_octave_start_pitch.pitch_number)

    @property
    def _one_line_menuing_summary(self):
        return '{} => {:d}'.format(
            self.source_pitch_range.one_line_named_pitch_repr,
            self.target_octave_start_pitch,
            )

    @property
    def _repr_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            is_indented=False,
            positional_argument_values=(
                self.source_pitch_range.one_line_named_pitch_repr,
                self.target_octave_start_pitch.pitch_number,
                ),
            )

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            positional_argument_values=(
                self.source_pitch_range,
                self.target_octave_start_pitch,
                ),
            )

    ### PUBLIC PROPERTIES ###

    @property
    def source_pitch_range(self):
        r'''Gets and sets source pitch range of mapping component.

        ::

            >>> mc.source_pitch_range
            PitchRange('[A0, C8]')

        Returns pitch range or none.
        '''
        return self._source_pitch_range

    @source_pitch_range.setter
    def source_pitch_range(self, source_pitch_range):
        self._source_pitch_range = PitchRange(source_pitch_range)

    @property
    def target_octave_start_pitch(self):
        r'''Gets and sets target octave start pitch of mapping component.

        ::

            >>> mc.target_octave_start_pitch
            NumberedPitch(15)

        Returns numbered pitch or none.
        '''
        return self._target_octave_start_pitch

    @target_octave_start_pitch.setter
    def target_octave_start_pitch(self, target_octave_start_pitch):
        self._target_octave_start_pitch = NumberedPitch(
            target_octave_start_pitch)



# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools.mathtools import NonreducedFraction
from abjad.tools import sequencetools


def test_sequencetools_repeat_sequence_to_weight_exactly_01():

    assert sequencetools.repeat_sequence_to_weight_exactly((5, -5, -5), 23) == (5, -5, -5, 5, -3)


def test_sequencetools_repeat_sequence_to_weight_exactly_02():
    r'''Works with nonreduced fractions.
    '''

    sequence = [NonreducedFraction(3, 16)]
    sequence = sequencetools.repeat_sequence_to_weight_exactly(sequence, NonreducedFraction(5, 4))

    assert sum(sequence) == NonreducedFraction(5, 4)
    assert [x.pair for x in sequence] == [(3, 16), (3, 16), (3, 16), (3, 16), (3, 16), (3, 16), (2, 16)]



# -*- encoding: utf-8 -*-


def all_are_unequal(expr):
    '''True when `expr` is a sequence all elements in `expr` are unequal:

    ::

        >>> sequencetools.all_are_unequal([1, 2, 3, 4, 9])
        True

    True when `expr` is an empty sequence:

    ::

        >>> sequencetools.all_are_unequal([])
        True

    Otherwise false:

    ::

        >>> sequencetools.all_are_unequal(17)
        False

    Returns boolean.
    '''

    try:
        return expr == type(expr)(set(expr))
    except TypeError:
        return False



# -*- encoding: utf-8 -*-
from abjad import *


def test_indicatortools_Tempo___init___01():
    r'''Initializes tempo with integer.
    '''

    tempo = Tempo(Duration(3, 32), 52)
    assert format(tempo, 'lilypond') == '\\tempo 16.=52'


def test_indicatortools_Tempo___init___02():
    r'''Initializes tempo with float.
    '''

    tempo = Tempo(Duration(3, 32), 52.5)
    assert format(tempo, 'lilypond') == '\\tempo 16.=52.5'


def test_indicatortools_Tempo___init___03():
    r'''Initializes tempo from tempo.
    '''

    tempo = Tempo(Duration(3, 32), 52)
    new = Tempo(tempo)

    assert tempo == new
    assert tempo is not new

    assert tempo.duration == new.duration
    assert tempo.duration is not new.duration


def test_indicatortools_Tempo___init___04():
    r'''Initializes tempo from integer pair.
    '''

    tempo = Tempo((3, 32), 52.5)
    assert format(tempo, 'lilypond') == '\\tempo 16.=52.5'


def test_indicatortools_Tempo___init___05():
    r'''Initializes tempo from textual indication.
    '''

    tempo = Tempo('Langsam')
    assert format(tempo, 'lilypond') == '\\tempo Langsam'


def test_indicatortools_Tempo___init___06():
    r'''Initializes tempo with tempo range.
    '''

    tempo = Tempo((1, 8), (52, 57.5))
    assert format(tempo, 'lilypond') == '\\tempo 8=52-57.5'


def test_indicatortools_Tempo___init___07():
    r'''Initializes tempo from text, duration and range.
    '''

    tempo = Tempo('Quick', Duration(1, 4), (120, 133))
    assert format(tempo, 'lilypond') == '\\tempo Quick 4=120-133'


def test_indicatortools_Tempo___init___08():
    r'''Initializes tempo from length-2 tuple.
    '''

    tempo = Tempo((Duration(1, 4), (120, 133)))
    assert format(tempo, 'lilypond') == '\\tempo 4=120-133'


def test_indicatortools_Tempo___init___09():
    r'''Initializes tempo from length-3 tuple.
    '''

    tempo = Tempo(('Quick', Duration(1, 4), (120, 133)))
    assert format(tempo, 'lilypond') == '\\tempo Quick 4=120-133'



# -*- encoding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import markuptools
from abjad.tools import scoretools
from abjad.tools import spannertools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import iterate


def label_leaves_in_expr_with_leaf_durations(
    expr,
    label_durations=True,
    label_written_durations=True,
    markup_direction=Down,
    ):
    r'''Label leaves in expression with leaf durations.
    
    ..  container:: example

        **Example 1.** Label leaves with written durations:

        ::

            >>> tuplet = Tuplet((2, 3), "c'8 d'8 e'8")
            >>> staff = scoretools.RhythmicStaff([tuplet])
            >>> override(staff).text_script.staff_padding = 2.5
            >>> override(staff).time_signature.stencil = False
            >>> labeltools.label_leaves_in_expr_with_leaf_durations(
            ...     tuplet, 
            ...     label_durations=False,
            ...     label_written_durations=True)

        ::

            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print format(staff)
            \new RhythmicStaff \with {
                \override TextScript #'staff-padding = #2.5
                \override TimeSignature #'stencil = ##f
            } {
                \times 2/3 {
                    c'8 _ \markup { \small 1/8 }
                    d'8 _ \markup { \small 1/8 }
                    e'8 _ \markup { \small 1/8 }
                }
            }

    ..  container:: example
    
        **Example 2.** Label leaves with actual durations:

        ::

            >>> tuplet = Tuplet((2, 3), "c'8 d'8 e'8")
            >>> staff = scoretools.RhythmicStaff([tuplet])
            >>> override(staff).text_script.staff_padding = 2.5
            >>> override(staff).time_signature.stencil = False
            >>> labeltools.label_leaves_in_expr_with_leaf_durations(
            ...     tuplet, 
            ...     label_durations=True,
            ...     label_written_durations=False)

        ::

            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print format(staff)
            \new RhythmicStaff \with {
                \override TextScript #'staff-padding = #2.5
                \override TimeSignature #'stencil = ##f
            } {
                \times 2/3 {
                    c'8 _ \markup { \small 1/12 }
                    d'8 _ \markup { \small 1/12 }
                    e'8 _ \markup { \small 1/12 }
                }
            }

    ..  container:: example

        **Example 3.** Label leaves in tuplet with both written and actual
        durations:

        ::

            >>> tuplet = Tuplet((2, 3), "c'8 d'8 e'8")
            >>> staff = scoretools.RhythmicStaff([tuplet])
            >>> override(staff).text_script.staff_padding = 2.5
            >>> override(staff).time_signature.stencil = False
            >>> labeltools.label_leaves_in_expr_with_leaf_durations(
            ...     tuplet, 
            ...     label_durations=True,
            ...     label_written_durations=True)

        ::

            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print format(staff)
            \new RhythmicStaff \with {
                \override TextScript #'staff-padding = #2.5
                \override TimeSignature #'stencil = ##f
            } {
                \times 2/3 {
                    c'8 _ \markup { \column { \small 1/8 \small 1/12 } }
                    d'8 _ \markup { \column { \small 1/8 \small 1/12 } }
                    e'8 _ \markup { \column { \small 1/8 \small 1/12 } }
                }
            }

    Returns none.
    '''

    prototype = (spannertools.Tie,)
    for leaf in iterate(expr).by_class(scoretools.Leaf):
        tie_spanners = leaf._get_spanners(prototype)
        if not tie_spanners:
            if leaf._get_indicators(durationtools.Multiplier):
                multiplier = leaf._get_indicator(durationtools.Multiplier)
                multiplier = '* {}'.format(multiplier)
            else:
                multiplier = ''
            if label_written_durations:
                label = markuptools.MarkupCommand(
                    'small', '{}{}'.format(
                        str(leaf.written_duration), multiplier))
                markup = markuptools.Markup(label, markup_direction)
                attach(markup, leaf)
            if label_durations:
                label = markuptools.MarkupCommand(
                    'small', str(leaf._get_duration()))
                markup = markuptools.Markup(label, markup_direction)
                attach(markup, leaf)
        elif tuple(tie_spanners)[0]._is_my_first_leaf(leaf):
            tie = tie_spanners.pop()
            if label_written_durations:
                written = sum([x.written_duration for x in tie])
                label = markuptools.MarkupCommand('small', str(written))
                markup = markuptools.Markup(label, markup_direction)
                attach(markup, leaf)
            if label_durations:
                duration = sum([x._get_duration() for x in tie])
                label = markuptools.MarkupCommand('small', str(duration))
                markup = markuptools.Markup(label, markup_direction)
                attach(markup, leaf)



# -*- encoding: utf-8 -*-
import inspect
import pytest
import abjad
from abjad.tools import *
pytest.skip()


classes = documentationtools.list_all_abjad_classes()
@pytest.mark.parametrize('class_', classes)
def test___init___01(class_):
    r'''All storage-formattable concrete classes initialize from empty input.
    '''

    if '_storage_format_specification' in dir(class_) and \
        not inspect.isabstract(class_):
        if hasattr(class_, '_default_positional_input_arguments'):
            args = class_._default_positional_input_arguments
            instance = class_(*args)
        else:
            instance = class_()



# -*- encoding: utf-8 -*-


def partition_integer_into_units(n):
    r'''Partitions positive integer into units:

    ::

        >>> mathtools.partition_integer_into_units(6)
        [1, 1, 1, 1, 1, 1]

    Partitions negative integer into units:

    ::

        >>> mathtools.partition_integer_into_units(-5)
        [-1, -1, -1, -1, -1]

    Partitions ``0`` into units:

    ::

        >>> mathtools.partition_integer_into_units(0)
        []

    Returns list of zero or more parts with absolute value equal to ``1``.
    '''
    from abjad.tools import mathtools

    result = abs(n) * [mathtools.sign(n) * 1]

    return result



# -*- encoding: utf-8 -*-
from abjad.tools.topleveltools import iterate


def set_written_pitch_of_pitched_components_in_expr(expr, written_pitch=0):
    r'''Set written pitch of pitched components in `expr` to `written_pitch`:

    ::

        >>> staff = Staff("c' d' e' f'")

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            c'4
            d'4
            e'4
            f'4
        }

    ::

        >>> pitchtools.set_written_pitch_of_pitched_components_in_expr(staff)

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            c'4
            c'4
            c'4
            c'4
        }

    Use as a way of neutralizing pitch information in an arbitrary piece of score.

    Returns none.
    '''
    from abjad.tools import scoretools

    for leaf in iterate(expr).by_class(scoretools.Leaf):
        if isinstance(leaf, scoretools.Note):
            leaf.written_pitch = written_pitch
        elif isinstance(leaf, scoretools.Chord):
            leaf.written_pitches = [written_pitch]



# -*- encoding: utf-8 -*-


def new(expr, *args, **kwargs):
    r'''Makes new `expr` with optionally new `args` and ``kwargs``.

    Returns new object with the same type as `expr`.
    '''

    if not hasattr(expr, '__makenew__'):
        message = 'does not implement the make-new protocol: {!r}.'
        message = message.format(expr)
        raise TypeError(message)

    new = expr.__makenew__(*args, **kwargs)
    return new



# -*- encoding: utf-8 -*-


def flatten_sequence(sequence, classes=None, depth=-1):
    '''Flattens `sequence`.

    ..  container:: example

        Faltten sequence completely:

        ::

            >>> sequence = [1, [2, 3, [4]], 5, [6, 7, [8]]]
            >>> sequencetools.flatten_sequence(sequence)
            [1, 2, 3, 4, 5, 6, 7, 8]

    ..  container:: example

        Flatten `sequence` to depth ``1``:

        ::

            >>> sequence = [1, [2, 3, [4]], 5, [6, 7, [8]]]
            >>> sequencetools.flatten_sequence(sequence, depth=1)
            [1, 2, 3, [4], 5, 6, 7, [8]]

    ..  container:: example

        Flatten `sequence` to depth ``2``:

        ::

            >>> sequence = [1, [2, 3, [4]], 5, [6, 7, [8]]]
            >>> sequencetools.flatten_sequence(sequence, depth=2)
            [1, 2, 3, 4, 5, 6, 7, 8]

    Leaves `sequence` unchanged.

    Returns new object of `sequence` type.
    '''
    from abjad.tools import selectiontools

    if classes is None:
        classes = (list, tuple, selectiontools.Selection)

    assert isinstance(sequence, classes), repr(sequence)
    sequence_type = type(sequence)
    return sequence_type(_flatten_helper(sequence, classes, depth))


# creates an iterator that can generate a flattened list,
# descending down into child elements to a depth given in the arguments.
# note that depth < 0 is effectively equivalent to infinity.
def _flatten_helper(sequence, classes, depth):
    if not isinstance(sequence, classes):
        yield sequence
    elif depth == 0:
        for i in sequence:
            yield i
    else:
        for i in sequence:
            # flatten an iterable by one level
            for j in _flatten_helper(i, classes, depth - 1):
                yield j



# -*- encoding: utf-8 -*-
import inspect
from abjad.tools.abctools import AbjadObject


class QuantizationJob(AbjadObject):
    r'''A copiable, picklable class for generating all ``QGrids`` which 
    are valid under a given ``SearchTree`` for a sequence 
    of ``QEventProxies``:

    ::

        >>> q_event_a = quantizationtools.PitchedQEvent(250, [0, 1])
        >>> q_event_b = quantizationtools.SilentQEvent(500)
        >>> q_event_c = quantizationtools.PitchedQEvent(750, [3, 7])
        >>> proxy_a = quantizationtools.QEventProxy(q_event_a, 0.25)
        >>> proxy_b = quantizationtools.QEventProxy(q_event_b, 0.5)
        >>> proxy_c = quantizationtools.QEventProxy(q_event_c, 0.75)

    ::

        >>> definition = {2: {2: None}, 3: None, 5: None}
        >>> search_tree = quantizationtools.UnweightedSearchTree(definition)

    ::

        >>> job = quantizationtools.QuantizationJob(
        ...     1, search_tree, [proxy_a, proxy_b, proxy_c])

    ``QuantizationJob`` generates ``QGrids`` when called, and stores those
    ``QGrids`` on its ``q_grids`` attribute, allowing them to be recalled 
    later, even if pickled:

    ::

        >>> job()
        >>> for q_grid in job.q_grids:
        ...     print q_grid.rtm_format
        1
        (1 (1 1 1 1 1))
        (1 (1 1 1))
        (1 (1 1))
        (1 ((1 (1 1)) (1 (1 1))))

    ``QuantizationJob`` is intended to be useful in 
    multiprocessing-enabled environments.

    Return ``QuantizationJob`` instance.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_job_id', 
        '_q_event_proxies', 
        '_q_grids', 
        '_search_tree',
        )

    ### INITIALIZER ###

    def __init__(
        self, 
        job_id=1, 
        search_tree=None, 
        q_event_proxies=None, 
        q_grids=None,
        ):
        from abjad.tools import quantizationtools

        search_tree = search_tree or \
            quantizationtools.UnweightedSearchTree()

        q_event_proxies = q_event_proxies or []

        assert isinstance(search_tree, quantizationtools.SearchTree)
        assert all(
            isinstance(x, quantizationtools.QEventProxy) 
            for x in q_event_proxies
            )
        self._job_id = job_id
        self._search_tree = search_tree
        self._q_event_proxies = tuple(q_event_proxies)
        if q_grids is None:
            self._q_grids = ()
        else:
            assert all(
                isinstance(x, quantizationtools.QGrid) 
                for x in q_grids
                )
            self._q_grids = tuple(q_grids)

    ### SPECIAL METHODS ###

    def __call__(self):
        r'''Calls quantization job.

        Returns none.
        '''
        from abjad.tools import quantizationtools
        q_grid = quantizationtools.QGrid()
        q_grid.fit_q_events(self.q_event_proxies)
        old_q_grids = []
        new_q_grids = [q_grid]
        while new_q_grids:
            q_grid = new_q_grids.pop()
            new_q_grids.extend(self.search_tree(q_grid))
            old_q_grids.append(q_grid)
        self._q_grids = tuple(old_q_grids)

    def __eq__(self, expr):
        r'''True when `expr` is a quantization job with job ID, search tree,
        q-event proxies and q-grids equal to those of this quantization job.
        Otherwise false.

        Returns boolean.
        '''
        if type(self) == type(expr):
            if self.job_id == expr.job_id:
                if self.search_tree == expr.search_tree:
                    if self.q_event_proxies == expr.q_event_proxies:
                        if self.q_grids == expr.q_grids:
                            return True
        return False

    def __getnewargs__(self):
        r'''Gets new arguments.

        Returns tuple.
        '''
        return (
            self.job_id, 
            self.search_tree, 
            self.q_event_proxies, 
            self.q_grids,
            )

    def __getstate__(self):
        r'''Gets state.

        Returns dictionary.
        '''
        return {
            '_job_id': self.job_id,
            '_q_event_proxies': self.q_event_proxies,
            '_q_grids': self.q_grids,
            '_search_tree': self.search_tree,
        }

    def __setstate__(self, state):
        r'''Sets state.

        Returns none.
        '''
        for key, value in state.iteritems():
            setattr(self, key, value)

    ### PUBLIC PROPERTIES ###

    @property
    def job_id(self):
        r'''The job id of the ``QuantizationJob``.

        ::

            >>> job.job_id
            1

        Only meaningful when the job is processed via multiprocessing,
        as the job id is necessary to reconstruct the order of jobs.

        Returns int.
        '''
        return self._job_id

    @property
    def q_event_proxies(self):
        r'''The ``QEventProxies`` the ``QuantizationJob`` was instantiated 
        with.

        ::

            >>> for q_event_proxy in job.q_event_proxies:
            ...     print format(q_event_proxy, 'storage')
            ...
            quantizationtools.QEventProxy(
                q_event=quantizationtools.PitchedQEvent(
                    offset=durationtools.Offset(250, 1),
                    pitches=(
                        pitchtools.NamedPitch("c'"),
                        pitchtools.NamedPitch("cs'"),
                        ),
                    ),
                offset=durationtools.Offset(1, 4),
                )
            quantizationtools.QEventProxy(
                q_event=quantizationtools.SilentQEvent(
                    offset=durationtools.Offset(500, 1),
                    ),
                offset=durationtools.Offset(1, 2),
                )
            quantizationtools.QEventProxy(
                q_event=quantizationtools.PitchedQEvent(
                    offset=durationtools.Offset(750, 1),
                    pitches=(
                        pitchtools.NamedPitch("ef'"),
                        pitchtools.NamedPitch("g'"),
                        ),
                    ),
                offset=durationtools.Offset(3, 4),
                )

        Returns tuple.
        '''
        return self._q_event_proxies

    @property
    def q_grids(self):
        r'''The generated ``QGrids``.

        ::

            >>> for q_grid in job.q_grids:
            ...     print q_grid.rtm_format
            1
            (1 (1 1 1 1 1))
            (1 (1 1 1))
            (1 (1 1))
            (1 ((1 (1 1)) (1 (1 1))))

        Returns tuple.
        '''
        return self._q_grids

    @property
    def search_tree(self):
        r'''The search tree the ``QuantizationJob`` was instantiated with.

        ::

            >>> job.search_tree
            UnweightedSearchTree(definition={2: {2: None}, 3: None, 5: None})

        Return ``SearchTree`` instance.
        '''
        return self._search_tree



# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_iterate_sequence_pairwise_wrapped_01():
    r'''Wrapped pairwise.
    '''

    sequence = range(6)
    pairs = sequencetools.iterate_sequence_pairwise_wrapped(sequence)
    assert list(pairs) == [(0, 1), (1, 2), (2, 3), (3, 4), (4, 5), (5, 0)]



# -*- encoding: utf-8 -*-
import collections
from abjad.tools import indicatortools
from abjad.tools import instrumenttools
from abjad.tools import scoretools
from abjad.tools import scoretools
from abjad.tools import scoretools
from abjad.tools.abctools.AbjadObject import AbjadObject
from abjad.tools.topleveltools import attach


class StringQuartetScoreTemplate(AbjadObject):
    '''String quartet score template.

    ::

        >>> template = templatetools.StringQuartetScoreTemplate()
        >>> score = template()

    ::

        >>> score
        Score-"String Quartet Score"<<1>>

    ..  doctest::

        >>> print format(score)
        \context Score = "String Quartet Score" <<
            \context StaffGroup = "String Quartet Staff Group" <<
                \context Staff = "First Violin Staff" {
                    \clef "treble"
                    \set Staff.instrumentName = \markup { Violin }
                    \set Staff.shortInstrumentName = \markup { Vn. }
                    \context Voice = "First Violin Voice" {
                    }
                }
                \context Staff = "Second Violin Staff" {
                    \clef "treble"
                    \set Staff.instrumentName = \markup { Violin }
                    \set Staff.shortInstrumentName = \markup { Vn. }
                    \context Voice = "Second Violin Voice" {
                    }
                }
                \context Staff = "Viola Staff" {
                    \clef "alto"
                    \set Staff.instrumentName = \markup { Viola }
                    \set Staff.shortInstrumentName = \markup { Va. }
                    \context Voice = "Viola Voice" {
                    }
                }
                \context Staff = "Cello Staff" {
                    \clef "bass"
                    \set Staff.instrumentName = \markup { Cello }
                    \set Staff.shortInstrumentName = \markup { Vc. }
                    \context Voice = "Cello Voice" {
                    }
                }
            >>
        >>

    Returns score template.
    '''

    ### CLASS VARIABLES ###

    context_name_abbreviations = collections.OrderedDict({
        'vn1': 'First Violin Voice',
        'vn2': 'Second Violin Voice',
        'va': 'Viola Voice',
        'vc': 'Cello Voice',
        })

    ### INITIALIZER ###

    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    def __call__(self):
        r'''Calls string quartet score template.

        Returns score.
        '''

        # make first violin voice and staff
        first_violin_voice = scoretools.Voice(name='First Violin Voice')
        first_violin_staff = scoretools.Staff(
            [first_violin_voice], name='First Violin Staff')
        clef = indicatortools.Clef('treble')
        attach(clef, first_violin_staff)
        violin = instrumenttools.Violin()
        attach(violin, first_violin_staff)

        # make second violin voice and staff
        second_violin_voice = scoretools.Voice(name='Second Violin Voice')
        second_violin_staff = scoretools.Staff(
            [second_violin_voice], name='Second Violin Staff')
        clef = indicatortools.Clef('treble')
        attach(clef, second_violin_staff)
        violin = instrumenttools.Violin()
        attach(violin, second_violin_staff)

        # make viola voice and staff
        viola_voice = scoretools.Voice(name='Viola Voice')
        viola_staff = scoretools.Staff(
            [viola_voice], name='Viola Staff')
        clef = indicatortools.Clef('alto')
        attach(clef, viola_staff)
        viola = instrumenttools.Viola()
        attach(viola, viola_staff)

        # make cello voice and staff
        cello_voice = scoretools.Voice(name='Cello Voice')
        cello_staff = scoretools.Staff(
            [cello_voice], name='Cello Staff')
        clef = indicatortools.Clef('bass')
        attach(clef, cello_staff)
        cello = instrumenttools.Cello()
        attach(cello, cello_staff)

        # make string quartet staff group
        string_quartet_staff_group = scoretools.StaffGroup([
            first_violin_staff,
            second_violin_staff,
            viola_staff,
            cello_staff,
            ],
            name='String Quartet Staff Group',
            )

        # make string quartet score
        string_quartet_score = scoretools.Score(
            [string_quartet_staff_group],
            name='String Quartet Score',
            )

        # return string quartet score
        return string_quartet_score



# -*- encoding: utf-8 -*-
import abc
import copy
from abjad.tools.abctools import AbjadObject


class GraphvizObject(AbjadObject):
    r'''An attributed Graphviz object.
    '''

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self, attributes=None):
        self._verify_attributes(attributes, '_attributes')

    ### SPECIAL METHODS ###

    def __getstate__(self):
        r'''Gets object state.
        '''
        return vars(self)

    ### PRIVATE METHODS ###

    def _format_attribute(self, name, value):
        return '{}={}'.format(name, self._format_value(value))

    def _format_attribute_list(self, attributes):
        result = []
        for k, v in sorted(attributes.items()):
            result.append(self._format_attribute(k, v))
        result[0] = '[' + result[0]
        result[-1] = result[-1] + '];'
        if 1 < len(result):
            for i, x in enumerate(result):
                if i < len(result) - 1:
                    result[i] = result[i] + ','
                if 0 < i:
                    result[i] = '\t' + result[i]
        return result

    def _format_value(self, value):
        if isinstance(value, bool):
            return repr(value).lower()
        elif isinstance(value, (int, float)):
            return repr(value)
        elif isinstance(value, str):
            if ' ' in value or \
                ',' in value or \
                '\\' in value or \
                "." in value or \
                '/' in value:
                return '"{}"'.format(value)
            return value
        elif isinstance(value, (list, tuple)):
            return '"{}"'.format(', '.join(
                self._format_value(x) for x in value))
        raise ValueError

    def _verify_attributes(self, attributes, destination):
        assert isinstance(attributes, (dict, type(None)))
        if attributes is None:
            attributes = {}
        else:
            attributes = copy.copy(attributes)
        setattr(self, destination, attributes)

    ### PUBLIC PROPERTIES ###

    @property
    def attributes(self):
        r'''Attributes of Graphviz object.
        '''
        return self._attributes



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools


def all_are_integer_equivalent_exprs(expr):
    '''True when `expr` is a sequence and all elements in `expr` are integer-equivalent expressions:

    ::

        >>> sequencetools.all_are_integer_equivalent_exprs([1, '2', 3.0, Fraction(4, 1)])
        True

    Otherwise false:

    ::

        >>> sequencetools.all_are_integer_equivalent_exprs([1, '2', 3.5, 4])
        False

    Returns boolean.
    '''

    try:
        return all(mathtools.is_integer_equivalent_expr(x) for x in expr)
    except TypeError:
        return False



# -*- encoding: utf-8 -*-
from abjad.tools.abctools.AbjadObject import AbjadObject


class BendAfter(AbjadObject):
    r'''A fall or doit.

    ::

        >>> note = Note("c'4")
        >>> bend = indicatortools.BendAfter(-4)
        >>> attach(bend, note)
        >>> show(note) # doctest: +SKIP

    ..  doctest::

        >>> print format(note)
        c'4 - \bendAfter #'-4.0

    '''

    ### CLASS VARIABLES ###

    _default_positional_input_arguments = (-4, )

    ### INITIALIZER ###

    def __init__(self, *args):
        self._format_slot = 'right'
        if len(args) == 1 and isinstance(args[0], type(self)):
            bend_amount = args[0].bend_amount
        elif len(args) == 1 and not isinstance(args[0], type(self)):
            bend_amount = args[0]
        else:
            message = 'can not initialize bend after from {!r}.'
            message = message.format(args)
            raise ValueError(message)
        bend_amount = float(bend_amount)
        self._bend_amount = bend_amount

    ### SPECIAL METHODS ###

    def __copy__(self, *args):
        r'''Copies bend after.

        Returns new bend after.
        '''
        new = type(self)(self.bend_amount)
        new.format_slot = self.format_slot
        return new

    def __eq__(self, expr):
        r'''True when `expr` is a bend after with bend amount equal to bend
        after. Otherwise false.

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            if self.bend_amount == expr.bend_amount:
                return True
        return False

    def __str__(self):
        r'''String representation of bend after.

        Returns string.
        '''
        return r"- \bendAfter #'{}".format(self.bend_amount)

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_repr_string(self):
        return str(self.bend_amount)

    @property
    def _lilypond_format(self):
        return str(self)

    ### PUBLIC PROPERTIES ###

    @property
    def bend_amount(self):
        r'''Amount of bend after.

        ::

            >>> bend.bend_amount
            -4.0

        Returns float.
        '''
        return self._bend_amount



# -*- encoding: utf-8 -*-
from abjad import *


def test_indicatortools_TempoInventory___contains___01():

    tempo_inventory = indicatortools.TempoInventory([
        (Duration(1, 8), 72), 
        ('Allegro', (1, 8), 84),
        ])

    assert Tempo((1, 8), 72) in tempo_inventory
    assert ((1, 8), 72) in tempo_inventory
    assert (Duration(1, 8), 72) in tempo_inventory
    assert ('Allegro', (1, 8), 84) in tempo_inventory
    assert ((1, 8), 96) not in tempo_inventory
